@@ -1,24 +1,18 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1999-2004
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 1999,2007 Oracle.  All rights reserved.
  *
- * $Id: bt_method.c,v 11.38 2004/09/22 03:31:26 bostic Exp $
+ * $Id: bt_method.c,v 12.8 2007/05/17 15:14:46 bostic Exp $
  */
 
 #include "db_config.h"
 
-#ifndef NO_SYSTEM_INCLUDES
-#include <sys/types.h>
-#endif
-
 #include "db_int.h"
 #include "dbinc/db_page.h"
 #include "dbinc/btree.h"
 #include "dbinc/qam.h"
 
-static int __bam_set_bt_maxkey __P((DB *, u_int32_t));
 static int __bam_set_bt_minkey __P((DB *, u_int32_t));
 static int __bam_set_bt_prefix
 	       __P((DB *, size_t(*)(DB *, const DBT *, const DBT *)));
@@ -52,7 +46,6 @@ __bam_db_create(dbp)
 	t->bt_prefix = __bam_defpfx;
 
 	dbp->set_bt_compare = __bam_set_bt_compare;
-	dbp->set_bt_maxkey = __bam_set_bt_maxkey;
 	dbp->get_bt_minkey = __bam_get_bt_minkey;
 	dbp->set_bt_minkey = __bam_set_bt_minkey;
 	dbp->set_bt_prefix = __bam_set_bt_prefix;
@@ -208,31 +201,6 @@ __bam_set_bt_compare(dbp, func)
 	return (0);
 }
 
-/*
- * __bam_set_bt_maxkey --
- *	Set the maximum keys per page.
- */
-static int
-__bam_set_bt_maxkey(dbp, bt_maxkey)
-	DB *dbp;
-	u_int32_t bt_maxkey;
-{
-	BTREE *t;
-
-	DB_ILLEGAL_AFTER_OPEN(dbp, "DB->set_bt_maxkey");
-	DB_ILLEGAL_METHOD(dbp, DB_OK_BTREE);
-
-	t = dbp->bt_internal;
-
-	if (bt_maxkey < 1) {
-		__db_err(dbp->dbenv, "minimum bt_maxkey value is 1");
-		return (EINVAL);
-	}
-
-	t->bt_maxkey = bt_maxkey;
-	return (0);
-}
-
 /*
  * __db_get_bt_minkey --
  *	Get the minimum keys per page.
@@ -270,7 +238,7 @@ __bam_set_bt_minkey(dbp, bt_minkey)
 	t = dbp->bt_internal;
 
 	if (bt_minkey < 2) {
-		__db_err(dbp->dbenv, "minimum bt_minkey value is 2");
+		__db_errx(dbp->dbenv, "minimum bt_minkey value is 2");
 		return (EINVAL);
 	}
 
