@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1996,2007 Oracle.  All rights reserved.
  *
- * $Id: bt_rec.c,v 12.21 2006/08/24 14:44:44 bostic Exp $
+ * $Id: bt_rec.c,v 12.28 2007/05/17 17:17:40 bostic Exp $
  */
 
 #include "db_config.h"
@@ -44,7 +43,6 @@ __bam_split_recover(dbenv, dbtp, lsnp, op, info)
 	u_int32_t ptype;
 	int cmp, l_update, p_update, r_update, rc, ret, rootsplit, t_ret;
 
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_split_print);
 
 	mpf = NULL;
@@ -141,19 +139,21 @@ do_left:	if (lp != NULL) {
 			goto out;
 
 		if (l_update) {
-			REC_DIRTY(mpf, &lp);
+			REC_DIRTY(mpf, file_dbp->priority, &lp);
 			memcpy(lp, _lp, file_dbp->pgsize);
 			lp->lsn = *lsnp;
-			if ((ret = __memp_fput(mpf, lp, 0)) != 0)
+			if ((ret =
+			    __memp_fput(mpf, lp, file_dbp->priority)) != 0)
 				goto out;
 			lp = NULL;
 		}
 
 		if (r_update) {
-			REC_DIRTY(mpf, &rp);
+			REC_DIRTY(mpf, file_dbp->priority, &rp);
 			memcpy(rp, _rp, file_dbp->pgsize);
 			rp->lsn = *lsnp;
-			if ((ret = __memp_fput(mpf, rp, 0)) != 0)
+			if ((ret =
+			    __memp_fput(mpf, rp, file_dbp->priority)) != 0)
 				goto out;
 			rp = NULL;
 		}
@@ -173,14 +173,15 @@ do_left:	if (lp != NULL) {
 				rc = 1;
 			}
 
-			REC_DIRTY(mpf, &pp);
+			REC_DIRTY(mpf, file_dbp->priority, &pp);
 			P_INIT(pp, file_dbp->pgsize, root_pgno,
 			    PGNO_INVALID, PGNO_INVALID, _lp->level + 1, ptype);
 			RE_NREC_SET(pp, rc ? __bam_total(file_dbp, _lp) +
 			    __bam_total(file_dbp, _rp) : 0);
 
 			pp->lsn = *lsnp;
-			if ((ret = __memp_fput(mpf, pp, 0)) != 0)
+			if ((ret =
+			    __memp_fput(mpf, pp, file_dbp->priority)) != 0)
 				goto out;
 			pp = NULL;
 		}
@@ -209,11 +210,11 @@ check_next:	/*
 			cmp = LOG_COMPARE(&LSN(np), &argp->nlsn);
 			CHECK_LSN(dbenv, op, cmp, &LSN(np), &argp->nlsn);
 			if (cmp == 0) {
-				REC_DIRTY(mpf, &np);
+				REC_DIRTY(mpf, file_dbp->priority, &np);
 				PREV_PGNO(np) = argp->right;
 				np->lsn = *lsnp;
-				if ((ret =
-				    __memp_fput(mpf, np, 0)) != 0)
+				if ((ret = __memp_fput(mpf,
+				    np, file_dbp->priority)) != 0)
 					goto out;
 				np = NULL;
 			}
@@ -232,9 +233,10 @@ check_next:	/*
 			goto lrundo;
 		}
 		if (LOG_COMPARE(lsnp, &LSN(pp)) == 0) {
-			REC_DIRTY(mpf, &pp);
+			REC_DIRTY(mpf, file_dbp->priority, &pp);
 			memcpy(pp, argp->pg.data, argp->pg.size);
-			if ((ret = __memp_fput(mpf, pp, 0)) != 0)
+			if ((ret =
+			    __memp_fput(mpf, pp, file_dbp->priority)) != 0)
 				goto out;
 			pp = NULL;
 		}
@@ -250,19 +252,19 @@ check_next:	/*
 lrundo:		if ((rootsplit && lp != NULL) || rp != NULL) {
 			if (rootsplit && lp != NULL &&
 			    LOG_COMPARE(lsnp, &LSN(lp)) == 0) {
-				REC_DIRTY(mpf, &lp);
+				REC_DIRTY(mpf, file_dbp->priority, &lp);
 				lp->lsn = argp->llsn;
-				if ((ret =
-				    __memp_fput(mpf, lp, 0)) != 0)
+				if ((ret = __memp_fput(mpf,
+				    lp, file_dbp->priority)) != 0)
 					goto out;
 				lp = NULL;
 			}
 			if (rp != NULL &&
 			    LOG_COMPARE(lsnp, &LSN(rp)) == 0) {
-				REC_DIRTY(mpf, &rp);
+				REC_DIRTY(mpf, file_dbp->priority, &rp);
 				rp->lsn = argp->rlsn;
-				if ((ret =
-				    __memp_fput(mpf, rp, 0)) != 0)
+				if ((ret = __memp_fput(mpf,
+				     rp, file_dbp->priority)) != 0)
 					goto out;
 				rp = NULL;
 			}
@@ -283,10 +285,10 @@ lrundo:		if ((rootsplit && lp != NULL) || rp != NULL) {
 				goto done;
 			}
 			if (LOG_COMPARE(lsnp, &LSN(np)) == 0) {
-				REC_DIRTY(mpf, &np);
+				REC_DIRTY(mpf, file_dbp->priority, &np);
 				PREV_PGNO(np) = argp->left;
 				np->lsn = argp->nlsn;
-				if (__memp_fput(mpf, np, 0))
+				if (__memp_fput(mpf, np, file_dbp->priority))
 					goto out;
 				np = NULL;
 			}
@@ -297,13 +299,17 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	/* Free any pages that weren't dirtied. */
-	if (pp != NULL && (t_ret = __memp_fput(mpf, pp, 0)) != 0 && ret == 0)
+	if (pp != NULL &&
+	    (t_ret = __memp_fput(mpf, pp, file_dbp->priority)) != 0 && ret == 0)
 		ret = t_ret;
-	if (lp != NULL && (t_ret = __memp_fput(mpf, lp, 0)) != 0 && ret == 0)
+	if (lp != NULL &&
+	    (t_ret = __memp_fput(mpf, lp, file_dbp->priority)) != 0 && ret == 0)
 		ret = t_ret;
-	if (np != NULL && (t_ret = __memp_fput(mpf, np, 0)) != 0 && ret == 0)
+	if (np != NULL &&
+	    (t_ret = __memp_fput(mpf, np, file_dbp->priority)) != 0 && ret == 0)
 		ret = t_ret;
-	if (rp != NULL && (t_ret = __memp_fput(mpf, rp, 0)) != 0 && ret == 0)
+	if (rp != NULL &&
+	    (t_ret = __memp_fput(mpf, rp, file_dbp->priority)) != 0 && ret == 0)
 		ret = t_ret;
 
 	/* Free any allocated space. */
@@ -343,7 +349,6 @@ __bam_rsplit_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, ret;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_rsplit_print);
 	REC_INTRO(__bam_rsplit_read, 1, 1);
 
@@ -373,7 +378,7 @@ __bam_rsplit_recover(dbenv, dbtp, lsnp, op, info)
 		 * that's not really necessary since the field is not used
 		 * otherwise.
 		 */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		rcnt = RE_NREC(pagep);
 		memcpy(pagep, argp->pgdbt.data, argp->pgdbt.size);
 		if (LEVEL(pagep) > LEAFLEVEL)
@@ -382,7 +387,7 @@ __bam_rsplit_recover(dbenv, dbtp, lsnp, op, info)
 		pagep->lsn = *lsnp;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Need to undo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		P_INIT(pagep, file_dbp->pgsize, root_pgno,
 		    argp->nrec, PGNO_INVALID, pagep->level + 1,
 		    IS_BTREE_PAGE(pagep) ? P_IBTREE : P_IRECNO);
@@ -391,7 +396,7 @@ __bam_rsplit_recover(dbenv, dbtp, lsnp, op, info)
 			goto out;
 		pagep->lsn = argp->rootlsn;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, dbc->priority)) != 0)
 		goto out;
 
 do_page:
@@ -418,14 +423,14 @@ do_page:
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &copy_lsn);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Need to redo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		pagep->lsn = *lsnp;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Need to undo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		memcpy(pagep, argp->pgdbt.data, argp->pgdbt.size);
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, dbc->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -433,7 +438,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, dbc->priority);
 	REC_CLOSE;
 }
 
@@ -460,7 +465,6 @@ __bam_adj_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, ret;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_adj_print);
 	REC_INTRO(__bam_adj_read, 1, 1);
 
@@ -483,7 +487,7 @@ __bam_adj_recover(dbenv, dbtp, lsnp, op, info)
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Need to redo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		if ((ret = __bam_adjindx(dbc,
 		    pagep, argp->indx, argp->indx_copy, argp->is_insert)) != 0)
 			goto out;
@@ -491,14 +495,14 @@ __bam_adj_recover(dbenv, dbtp, lsnp, op, info)
 		LSN(pagep) = *lsnp;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Need to undo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		if ((ret = __bam_adjindx(dbc,
 		    pagep, argp->indx, argp->indx_copy, !argp->is_insert)) != 0)
 			goto out;
 
 		LSN(pagep) = argp->lsn;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, dbc->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -506,7 +510,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, dbc->priority);
 	REC_CLOSE;
 }
 
@@ -534,7 +538,6 @@ __bam_cadjust_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, ret;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_cadjust_print);
 	REC_INTRO(__bam_cadjust_read, 1, 0);
 
@@ -557,7 +560,7 @@ __bam_cadjust_recover(dbenv, dbtp, lsnp, op, info)
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Need to redo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		if (IS_BTREE_PAGE(pagep)) {
 			GET_BINTERNAL(file_dbp, pagep, argp->indx)->nrecs +=
 			    argp->adjust;
@@ -573,7 +576,7 @@ __bam_cadjust_recover(dbenv, dbtp, lsnp, op, info)
 		LSN(pagep) = *lsnp;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Need to undo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		if (IS_BTREE_PAGE(pagep)) {
 			GET_BINTERNAL(file_dbp, pagep, argp->indx)->nrecs -=
 			    argp->adjust;
@@ -587,7 +590,7 @@ __bam_cadjust_recover(dbenv, dbtp, lsnp, op, info)
 		}
 		LSN(pagep) = argp->lsn;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -595,7 +598,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, file_dbp->priority);
 	REC_CLOSE;
 }
 
@@ -623,7 +626,6 @@ __bam_cdel_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, ret;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_cdel_print);
 	REC_INTRO(__bam_cdel_read, 1, 0);
 
@@ -646,14 +648,14 @@ __bam_cdel_recover(dbenv, dbtp, lsnp, op, info)
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Need to redo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		indx = argp->indx + (TYPE(pagep) == P_LBTREE ? O_INDX : 0);
 		B_DSET(GET_BKEYDATA(file_dbp, pagep, indx)->type);
 
 		LSN(pagep) = *lsnp;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Need to undo update described. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		indx = argp->indx + (TYPE(pagep) == P_LBTREE ? O_INDX : 0);
 		B_DCLR(GET_BKEYDATA(file_dbp, pagep, indx)->type);
 
@@ -663,7 +665,7 @@ __bam_cdel_recover(dbenv, dbtp, lsnp, op, info)
 
 		LSN(pagep) = argp->lsn;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -671,7 +673,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, file_dbp->priority);
 	REC_CLOSE;
 }
 
@@ -701,7 +703,6 @@ __bam_repl_recover(dbenv, dbtp, lsnp, op, info)
 	u_int8_t *p;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_repl_print);
 	REC_INTRO(__bam_repl_read, 1, 1);
 
@@ -729,7 +730,7 @@ __bam_repl_recover(dbenv, dbtp, lsnp, op, info)
 		 *
 		 * Re-build the replacement item.
 		 */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		memset(&dbt, 0, sizeof(dbt));
 		dbt.size = argp->prefix + argp->suffix + argp->repl.size;
 		if ((ret = __os_malloc(dbenv, dbt.size, &dbt.data)) != 0)
@@ -753,7 +754,7 @@ __bam_repl_recover(dbenv, dbtp, lsnp, op, info)
 		 *
 		 * Re-build the original item.
 		 */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		memset(&dbt, 0, sizeof(dbt));
 		dbt.size = argp->prefix + argp->suffix + argp->orig.size;
 		if ((ret = __os_malloc(dbenv, dbt.size, &dbt.data)) != 0)
@@ -776,7 +777,7 @@ __bam_repl_recover(dbenv, dbtp, lsnp, op, info)
 
 		LSN(pagep) = argp->lsn;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, dbc->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -784,7 +785,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, dbc->priority);
 	REC_CLOSE;
 }
 
@@ -811,7 +812,6 @@ __bam_root_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, ret;
 
 	meta = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_root_print);
 	REC_INTRO(__bam_root_read, 0, 0);
 
@@ -833,16 +833,16 @@ __bam_root_recover(dbenv, dbtp, lsnp, op, info)
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(meta), &argp->meta_lsn);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Need to redo update described. */
-		REC_DIRTY(mpf, &meta);
+		REC_DIRTY(mpf, file_dbp->priority, &meta);
 		meta->root = argp->root_pgno;
 		meta->dbmeta.lsn = *lsnp;
 		((BTREE *)file_dbp->bt_internal)->bt_root = meta->root;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Nothing to undo except lsn. */
-		REC_DIRTY(mpf, &meta);
+		REC_DIRTY(mpf, file_dbp->priority, &meta);
 		meta->dbmeta.lsn = argp->meta_lsn;
 	}
-	if ((ret = __memp_fput(mpf, meta, 0)) != 0)
+	if ((ret = __memp_fput(mpf, meta, file_dbp->priority)) != 0)
 		goto out;
 	meta = NULL;
 
@@ -850,7 +850,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (meta != NULL)
-		(void)__memp_fput(mpf, meta, 0);
+		(void)__memp_fput(mpf, meta, file_dbp->priority);
 	REC_CLOSE;
 }
 
@@ -876,7 +876,6 @@ __bam_curadj_recover(dbenv, dbtp, lsnp, op, info)
 	DB_MPOOLFILE *mpf;
 	int ret;
 
-	COMPQUIET(info, NULL);
 	COMPQUIET(mpf, NULL);
 
 	REC_PRINT(__bam_curadj_print);
@@ -938,7 +937,6 @@ __bam_rcuradj_recover(dbenv, dbtp, lsnp, op, info)
 	DB_MPOOLFILE *mpf;
 	int ret, t_ret;
 
-	COMPQUIET(info, NULL);
 	COMPQUIET(mpf, NULL);
 	rdbc = NULL;
 
@@ -995,7 +993,7 @@ __bam_rcuradj_recover(dbenv, dbtp, lsnp, op, info)
 	}
 
 done:	*lsnp = argp->prev_lsn;
-out:	if (rdbc != NULL && (t_ret = __db_c_close(rdbc)) != 0 && ret == 0)
+out:	if (rdbc != NULL && (t_ret = __dbc_close(rdbc)) != 0 && ret == 0)
 		ret = t_ret;
 	REC_CLOSE;
 }
@@ -1023,7 +1021,6 @@ __bam_relink_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, ret;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_relink_print);
 	REC_INTRO(__bam_relink_read, 1, 0);
 
@@ -1051,7 +1048,7 @@ __bam_relink_recover(dbenv, dbtp, lsnp, op, info)
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn_next);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Redo the remove or replace. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		if (argp->new_pgno == PGNO_INVALID)
 			pagep->prev_pgno = argp->prev;
 		else
@@ -1060,13 +1057,13 @@ __bam_relink_recover(dbenv, dbtp, lsnp, op, info)
 		pagep->lsn = *lsnp;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Undo the remove or replace. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->prev_pgno = argp->pgno;
 
 		pagep->lsn = argp->lsn_next;
 	}
 
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -1087,7 +1084,7 @@ prev:	if ((ret = __memp_fget(mpf, &argp->prev, NULL,
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn_prev);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Redo the relink. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		if (argp->new_pgno == PGNO_INVALID)
 			pagep->next_pgno = argp->next;
 		else
@@ -1096,12 +1093,12 @@ prev:	if ((ret = __memp_fget(mpf, &argp->prev, NULL,
 		pagep->lsn = *lsnp;
 	} else if (LOG_COMPARE(lsnp, &LSN(pagep)) == 0 && DB_UNDO(op)) {
 		/* Undo the relink. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->next_pgno = argp->pgno;
 		pagep->lsn = argp->lsn_prev;
 	}
 
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -1109,7 +1106,7 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, file_dbp->priority);
 	REC_CLOSE;
 }
 
@@ -1139,8 +1136,6 @@ __bam_merge_recover(dbenv, dbtp, lsnp, op, info)
 	u_int8_t *bp;
 	int cmp_n, cmp_p, i, ret;
 
-	COMPQUIET(info, NULL);
-
 	REC_PRINT(__bam_merge_print);
 	REC_INTRO(__bam_merge_read, 1, 1);
 
@@ -1167,7 +1162,7 @@ __bam_merge_recover(dbenv, dbtp, lsnp, op, info)
 		 * needed data.
 		 */
 		DB_ASSERT(dbenv, argp->hdr.size == 0 || NUM_ENT(pagep) == 0);
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		if (argp->hdr.size != 0) {
 			P_INIT(pagep, file_dbp->pgsize, pagep->pgno,
 			     PREV_PGNO(argp->hdr.data),
@@ -1203,7 +1198,7 @@ __bam_merge_recover(dbenv, dbtp, lsnp, op, info)
 		 * the proper number of items from the logical end
 		 * of the page.
 		 */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		for (i = 0; i < (int)(argp->ind.size / sizeof(*ninp)); i++) {
 			indx = NUM_ENT(pagep) - 1;
 			if (P_INP(file_dbp, pagep)[indx] ==
@@ -1240,7 +1235,7 @@ __bam_merge_recover(dbenv, dbtp, lsnp, op, info)
 		pagep->lsn = argp->lsn;
 	}
 
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, dbc->priority)) != 0)
 		goto out;
 
 next:	if ((ret = __memp_fget(mpf, &argp->npgno, NULL,
@@ -1262,13 +1257,13 @@ next:	if ((ret = __memp_fget(mpf, &argp->npgno, NULL,
 
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Need to truncate the page. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		HOFFSET(pagep) = file_dbp->pgsize;
 		NUM_ENT(pagep) = 0;
 		pagep->lsn = *lsnp;
 	} else if (cmp_n == 0 && !DB_REDO(op)) {
 		/* Need to put the data back on the page. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, dbc->priority, &pagep);
 		if (TYPE(pagep) == P_OVERFLOW) {
 			OV_REF(pagep) = OV_REF(argp->hdr.data);
 			OV_LEN(pagep) = OV_LEN(argp->hdr.data);
@@ -1291,7 +1286,7 @@ next:	if ((ret = __memp_fget(mpf, &argp->npgno, NULL,
 		pagep->lsn = argp->nlsn;
 	}
 
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, dbc->priority)) != 0)
 		goto out;
 done:
 	*lsnp = argp->prev_lsn;
@@ -1324,8 +1319,6 @@ __bam_pgno_recover(dbenv, dbtp, lsnp, op, info)
 	db_pgno_t *pgnop;
 	int cmp_n, cmp_p, ret;
 
-	COMPQUIET(info, NULL);
-
 	REC_PRINT(__bam_pgno_print);
 	REC_INTRO(__bam_pgno_read, 1, 0);
 
@@ -1352,7 +1345,8 @@ __bam_pgno_recover(dbenv, dbtp, lsnp, op, info)
 					     &((BOVERFLOW *)(bi->data))->pgno;
 				else
 					pgnop = &bi->pgno;
-				if ((ret = __memp_fput(mpf, npagep, 0)) != 0)
+				if ((ret = __memp_fput(mpf,
+				    npagep, file_dbp->priority)) != 0)
 					goto out;
 				break;
 			}
@@ -1370,17 +1364,17 @@ __bam_pgno_recover(dbenv, dbtp, lsnp, op, info)
 
 		if (DB_REDO(op)) {
 			/* Need to redo update described. */
-			REC_DIRTY(mpf, &pagep);
+			REC_DIRTY(mpf, file_dbp->priority, &pagep);
 			*pgnop = argp->npgno;
 			pagep->lsn = *lsnp;
 		} else {
-			REC_DIRTY(mpf, &pagep);
+			REC_DIRTY(mpf, file_dbp->priority, &pagep);
 			*pgnop = argp->opgno;
 			pagep->lsn = argp->lsn;
 		}
 	}
 
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 
 done:
@@ -1413,7 +1407,6 @@ __bam_relink_43_recover(dbenv, dbtp, lsnp, op, info)
 	int cmp_n, cmp_p, modified, ret;
 
 	pagep = NULL;
-	COMPQUIET(info, NULL);
 	REC_PRINT(__bam_relink_43_print);
 	REC_INTRO(__bam_relink_43_read, 1, 0);
 
@@ -1434,23 +1427,21 @@ __bam_relink_43_recover(dbenv, dbtp, lsnp, op, info)
 		} else
 			goto next2;
 	}
-	modified = 0;
 
 	cmp_p = LOG_COMPARE(&LSN(pagep), &argp->lsn);
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Redo the relink. */
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->lsn = *lsnp;
-		modified = 1;
 	} else if (LOG_COMPARE(lsnp, &LSN(pagep)) == 0 && DB_UNDO(op)) {
 		/* Undo the relink. */
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->next_pgno = argp->next;
 		pagep->prev_pgno = argp->prev;
-
 		pagep->lsn = argp->lsn;
-		modified = 1;
 	}
-	if ((ret = __memp_fput(mpf, pagep, modified ? DB_MPOOL_DIRTY : 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -1472,12 +1463,12 @@ next2: if ((ret = __memp_fget(mpf, &argp->next, NULL, 0, &pagep)) != 0) {
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn_next);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Redo the remove or undo the add. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->prev_pgno = argp->prev;
 		modified = 1;
 	} else if (cmp_n == 0 && DB_UNDO(op)) {
 		/* Undo the remove or redo the add. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->prev_pgno = argp->pgno;
 		modified = 1;
 	}
@@ -1487,7 +1478,7 @@ next2: if ((ret = __memp_fget(mpf, &argp->next, NULL, 0, &pagep)) != 0) {
 		else
 			pagep->lsn = *lsnp;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -1508,12 +1499,12 @@ prev: if ((ret = __memp_fget(mpf, &argp->prev, NULL, 0, &pagep)) != 0) {
 	CHECK_LSN(dbenv, op, cmp_p, &LSN(pagep), &argp->lsn_prev);
 	if (cmp_p == 0 && DB_REDO(op)) {
 		/* Redo the relink. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->next_pgno = argp->next;
 		modified = 1;
 	} else if (LOG_COMPARE(lsnp, &LSN(pagep)) == 0 && DB_UNDO(op)) {
 		/* Undo the relink. */
-		REC_DIRTY(mpf, &pagep);
+		REC_DIRTY(mpf, file_dbp->priority, &pagep);
 		pagep->next_pgno = argp->pgno;
 		modified = 1;
 	}
@@ -1523,7 +1514,7 @@ prev: if ((ret = __memp_fget(mpf, &argp->prev, NULL, 0, &pagep)) != 0) {
 		else
 			pagep->lsn = *lsnp;
 	}
-	if ((ret = __memp_fput(mpf, pagep, 0)) != 0)
+	if ((ret = __memp_fput(mpf, pagep, file_dbp->priority)) != 0)
 		goto out;
 	pagep = NULL;
 
@@ -1531,6 +1522,6 @@ done:	*lsnp = argp->prev_lsn;
 	ret = 0;
 
 out:	if (pagep != NULL)
-		(void)__memp_fput(mpf, pagep, 0);
+		(void)__memp_fput(mpf, pagep, file_dbp->priority);
 	REC_CLOSE;
 }
