@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1996,2007 Oracle.  All rights reserved.
  *
- * $Id: db_dump.c,v 12.9 2006/08/26 09:23:01 bostic Exp $
+ * $Id: db_dump.c,v 12.15 2007/05/17 15:14:59 bostic Exp $
  */
 
 #include "db_config.h"
@@ -15,12 +14,11 @@
 
 #ifndef lint
 static const char copyright[] =
-    "Copyright (c) 1996-2006\nOracle Corporation.  All rights reserved.\n";
+    "Copyright (c) 1996,2007 Oracle.  All rights reserved.\n";
 #endif
 
 int	 db_init __P((DB_ENV *, char *, int, u_int32_t, int *));
 int	 dump_sub __P((DB_ENV *, DB *, char *, int, int));
-int	 is_sub __P((DB *, int *));
 int	 main __P((int, char *[]));
 int	 show_subs __P((DB *));
 int	 usage __P((void));
@@ -40,10 +38,10 @@ main(argc, argv)
 	u_int32_t cache;
 	int ch;
 	int exitval, keyflag, lflag, nflag, pflag, private;
-	int ret, Rflag, rflag, resize, subs;
+	int ret, Rflag, rflag, resize;
 	char *dopt, *home, *passwd, *subname;
 
-	if ((progname = strrchr(argv[0], '/')) == NULL)
+	if ((progname = __db_rpath(argv[0])) == NULL)
 		progname = argv[0];
 	else
 		++progname;
@@ -225,20 +223,16 @@ retry:	if ((ret = db_env_create(&dbenv, 0)) != 0) {
 			goto err;
 		}
 	} else if (lflag) {
-		if (is_sub(dbp, &subs))
-			goto err;
-		if (subs == 0) {
+		if (dbp->get_multiple(dbp)) {
+			if (show_subs(dbp))
+				goto err;
+		} else {
 			dbp->errx(dbp,
 			    "%s: does not contain multiple databases", argv[0]);
 			goto err;
 		}
-		if (show_subs(dbp))
-			goto err;
 	} else {
-		subs = 0;
-		if (subname == NULL && is_sub(dbp, &subs))
-			goto err;
-		if (subs) {
+		if (subname == NULL && dbp->get_multiple(dbp)) {
 			if (dump_sub(dbenv, dbp, argv[0], pflag, keyflag))
 				goto err;
 		} else
@@ -329,47 +323,6 @@ err:	dbenv->err(dbenv, ret, "DB_ENV->open");
 	return (1);
 }
 
-/*
- * is_sub --
- *	Return if the database contains subdatabases.
- */
-int
-is_sub(dbp, yesno)
-	DB *dbp;
-	int *yesno;
-{
-	DB_BTREE_STAT *btsp;
-	DB_HASH_STAT *hsp;
-	int ret;
-
-	switch (dbp->type) {
-	case DB_BTREE:
-	case DB_RECNO:
-		if ((ret = dbp->stat(dbp, NULL, &btsp, DB_FAST_STAT)) != 0) {
-			dbp->err(dbp, ret, "DB->stat");
-			return (ret);
-		}
-		*yesno = btsp->bt_metaflags & BTM_SUBDB ? 1 : 0;
-		free(btsp);
-		break;
-	case DB_HASH:
-		if ((ret = dbp->stat(dbp, NULL, &hsp, DB_FAST_STAT)) != 0) {
-			dbp->err(dbp, ret, "DB->stat");
-			return (ret);
-		}
-		*yesno = hsp->hash_metaflags & DB_HASH_SUBDB ? 1 : 0;
-		free(hsp);
-		break;
-	case DB_QUEUE:
-		break;
-	case DB_UNKNOWN:
-	default:
-		dbp->errx(dbp, "unknown database type");
-		return (1);
-	}
-	return (0);
-}
-
 /*
  * dump_sub --
  *	Dump out the records for a DB containing subdatabases.
@@ -398,7 +351,8 @@ dump_sub(dbenv, parent_dbp, parent_name, pflag, keyflag)
 
 	memset(&key, 0, sizeof(key));
 	memset(&data, 0, sizeof(data));
-	while ((ret = dbcp->c_get(dbcp, &key, &data, DB_NEXT)) == 0) {
+	while ((ret = dbcp->get(dbcp, &key, &data,
+	    DB_IGNORE_LEASE | DB_NEXT)) == 0) {
 		/* Nul terminate the subdatabase name. */
 		if ((subdb = malloc(key.size + 1)) == NULL) {
 			dbenv->err(dbenv, ENOMEM, NULL);
@@ -430,7 +384,7 @@ dump_sub(dbenv, parent_dbp, parent_name, pflag, keyflag)
 		return (1);
 	}
 
-	if ((ret = dbcp->c_close(dbcp)) != 0) {
+	if ((ret = dbcp->close(dbcp)) != 0) {
 		parent_dbp->err(parent_dbp, ret, "DBcursor->close");
 		return (1);
 	}
@@ -461,7 +415,8 @@ show_subs(dbp)
 
 	memset(&key, 0, sizeof(key));
 	memset(&data, 0, sizeof(data));
-	while ((ret = dbcp->c_get(dbcp, &key, &data, DB_NEXT)) == 0) {
+	while ((ret = dbcp->get(dbcp, &key, &data,
+	    DB_IGNORE_LEASE | DB_NEXT)) == 0) {
 		if ((ret = dbp->dbenv->prdbt(
 		    &key, 1, NULL, stdout, __db_pr_callback, 0)) != 0) {
 			dbp->errx(dbp, NULL);
@@ -473,7 +428,7 @@ show_subs(dbp)
 		return (1);
 	}
 
-	if ((ret = dbcp->c_close(dbcp)) != 0) {
+	if ((ret = dbcp->close(dbcp)) != 0) {
 		dbp->err(dbp, ret, "DBcursor->close");
 		return (1);
 	}
