@@ -1,33 +1,22 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2004
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 1996,2007 Oracle.  All rights reserved.
  *
- * $Id: db_load.c,v 11.100 2004/10/29 17:29:02 bostic Exp $
+ * $Id: db_load.c,v 12.23 2007/05/17 17:17:42 bostic Exp $
  */
 
 #include "db_config.h"
 
-#ifndef lint
-static const char copyright[] =
-    "Copyright (c) 1996-2004\nSleepycat Software Inc.  All rights reserved.\n";
-#endif
-
-#ifndef NO_SYSTEM_INCLUDES
-#include <sys/types.h>
-
-#include <limits.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#endif
-
 #include "db_int.h"
 #include "dbinc/db_page.h"
 #include "dbinc/db_am.h"
 
+#ifndef lint
+static const char copyright[] =
+    "Copyright (c) 1996,2007 Oracle.  All rights reserved.\n";
+#endif
+
 typedef struct {			/* XXX: Globals. */
 	const char *progname;		/* Program name. */
 	char	*hdrbuf;		/* Input file header. */
@@ -42,7 +31,7 @@ typedef struct {			/* XXX: Globals. */
 	u_int32_t cache;		/* Env cache size. */
 } LDG;
 
-void	badend __P((DB_ENV *));
+int	badend __P((DB_ENV *));
 void	badnum __P((DB_ENV *));
 int	configure __P((DB_ENV *, DB *, char **, char **, int *));
 int	convprintable __P((DB_ENV *, char *, char **));
@@ -51,13 +40,14 @@ int	dbt_rdump __P((DB_ENV *, DBT *));
 int	dbt_rprint __P((DB_ENV *, DBT *));
 int	dbt_rrecno __P((DB_ENV *, DBT *, int));
 int	dbt_to_recno __P((DB_ENV *, DBT *, db_recno_t *));
-int	digitize __P((DB_ENV *, int, int *));
 int	env_create __P((DB_ENV **, LDG *));
 int	load __P((DB_ENV *, char *, DBTYPE, char **, u_int, LDG *, int *));
 int	main __P((int, char *[]));
 int	rheader __P((DB_ENV *, DB *, DBTYPE *, char **, int *, int *));
 int	usage __P((void));
-int	version_check __P((const char *));
+int	version_check __P((void));
+
+const char *progname;
 
 #define	G(f)	((LDG *)dbenv->app_private)->f
 
@@ -71,7 +61,7 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	enum { NOTSET, FILEID_RESET, LSN_RESET, INVALID } reset;
+	enum { NOTSET, FILEID_RESET, LSN_RESET, STANDARD_LOAD } mode;
 	extern char *optarg;
 	extern int optind;
 	DBTYPE dbtype;
@@ -81,7 +71,15 @@ main(argc, argv)
 	int ch, existed, exitval, ret;
 	char **clist, **clp;
 
-	ldg.progname = "db_load";
+	if ((progname = __db_rpath(argv[0])) == NULL)
+		progname = argv[0];
+	else
+		++progname;
+
+	if ((ret = version_check()) != 0)
+		return (ret);
+
+	ldg.progname = progname;
 	ldg.lineno = 0;
 	ldg.endodata = ldg.endofile = 0;
 	ldg.version = 1;
@@ -90,10 +88,7 @@ main(argc, argv)
 	ldg.home = NULL;
 	ldg.passwd = NULL;
 
-	if ((ret = version_check(ldg.progname)) != 0)
-		return (ret);
-
-	reset = NOTSET;
+	mode = NOTSET;
 	ldf = 0;
 	exitval = existed = 0;
 	dbtype = DB_UNKNOWN;
@@ -115,16 +110,16 @@ main(argc, argv)
 	while ((ch = getopt(argc, argv, "c:f:h:nP:r:Tt:V")) != EOF)
 		switch (ch) {
 		case 'c':
-			if (reset != NOTSET)
+			if (mode != NOTSET && mode != STANDARD_LOAD)
 				return (usage());
-			reset = INVALID;
+			mode = STANDARD_LOAD;
 
 			*clp++ = optarg;
 			break;
 		case 'f':
-			if (reset != NOTSET)
+			if (mode != NOTSET && mode != STANDARD_LOAD)
 				return (usage());
-			reset = INVALID;
+			mode = STANDARD_LOAD;
 
 			if (freopen(optarg, "r", stdin) == NULL) {
 				fprintf(stderr, "%s: %s: reopen: %s\n",
@@ -136,9 +131,9 @@ main(argc, argv)
 			ldg.home = optarg;
 			break;
 		case 'n':
-			if (reset != NOTSET)
+			if (mode != NOTSET && mode != STANDARD_LOAD)
 				return (usage());
-			reset = INVALID;
+			mode = STANDARD_LOAD;
 
 			ldf |= LDF_NOOVERWRITE;
 			break;
@@ -153,26 +148,26 @@ main(argc, argv)
 			ldf |= LDF_PASSWORD;
 			break;
 		case 'r':
-			if (reset == INVALID)
+			if (mode == STANDARD_LOAD)
 				return (usage());
 			if (strcmp(optarg, "lsn") == 0)
-				reset = LSN_RESET;
+				mode = LSN_RESET;
 			else if (strcmp(optarg, "fileid") == 0)
-				reset = FILEID_RESET;
+				mode = FILEID_RESET;
 			else
 				return (usage());
 			break;
 		case 'T':
-			if (reset != NOTSET)
+			if (mode != NOTSET && mode != STANDARD_LOAD)
 				return (usage());
-			reset = INVALID;
+			mode = STANDARD_LOAD;
 
 			ldf |= LDF_NOHEADER;
 			break;
 		case 't':
-			if (reset != NOTSET)
+			if (mode != NOTSET && mode != STANDARD_LOAD)
 				return (usage());
-			reset = INVALID;
+			mode = STANDARD_LOAD;
 
 			if (strcmp(optarg, "btree") == 0) {
 				dbtype = DB_BTREE;
@@ -215,16 +210,17 @@ main(argc, argv)
 		goto shutdown;
 
 	/* If we're resetting the LSNs, that's an entirely separate path. */
-	switch (reset) {
+	switch (mode) {
 	case FILEID_RESET:
 		exitval = dbenv->fileid_reset(
-		    dbenv, argv[0], ldf & LDF_PASSWORD ? 1 : 0);
+		    dbenv, argv[0], ldf & LDF_PASSWORD ? DB_ENCRYPT : 0);
 		break;
 	case LSN_RESET:
 		exitval = dbenv->lsn_reset(
-		    dbenv, argv[0], ldf & LDF_PASSWORD ? 1 : 0);
+		    dbenv, argv[0], ldf & LDF_PASSWORD ? DB_ENCRYPT : 0);
 		break;
-	default:
+	case NOTSET:
+	case STANDARD_LOAD:
 		while (!ldg.endofile)
 			if (load(dbenv, argv[0], dbtype, clist, ldf,
 			    &ldg, &existed) != 0)
@@ -400,7 +396,7 @@ retry_db:
 	/* Open the DB file. */
 	if ((ret = dbp->open(dbp, NULL, name, subdb, dbtype,
 	    DB_CREATE | (TXN_ON(dbenv) ? DB_AUTO_COMMIT : 0),
-	    __db_omode("rwrwrw"))) != 0) {
+	    __db_omode("rw-rw-rw-"))) != 0) {
 		dbp->err(dbp, ret, "DB->open: %s", name);
 		goto err;
 	}
@@ -528,7 +524,7 @@ retry:		if (txn != NULL)
 		}
 	}
 done:	rval = 0;
-	DB_ASSERT(ctxn == NULL);
+	DB_ASSERT(dbenv, ctxn == NULL);
 	if (txn != NULL && (ret = txn->commit(txn, 0)) != 0) {
 		txn = NULL;
 		goto err;
@@ -536,7 +532,7 @@ done:	rval = 0;
 
 	if (0) {
 err:		rval = 1;
-		DB_ASSERT(ctxn == NULL);
+		DB_ASSERT(dbenv, ctxn == NULL);
 		if (txn != NULL)
 			(void)txn->abort(txn);
 	}
@@ -651,23 +647,22 @@ err:	dbenv->err(dbenv, ret, "DB_ENV->open");
 			if ((ret = dbp->set_flags(dbp, flag)) != 0) {	\
 				dbp->err(dbp, ret, "%s: set_flags: %s",	\
 				    G(progname), name);			\
-				return (1);				\
+				goto err;				\
 			}						\
 			break;						\
 		case '0':						\
 			break;						\
 		default:						\
 			badnum(dbenv);					\
-			return (1);					\
+			goto err;					\
 		}							\
 		continue;						\
 	}
 #define	NUMBER(name, value, keyword, func, t)				\
 	if (strcmp(name, keyword) == 0) {				\
-		if (__db_getlong(dbenv,					\
-		    NULL, value, 1, LONG_MAX, &val) != 0)		\
-			return (1);					\
-		if ((ret = dbp->func(dbp, (t)val)) != 0)		\
+		if ((ret = __db_getlong(dbenv,				\
+		    NULL, value, 0, LONG_MAX, &val)) != 0 ||		\
+		    (ret = dbp->func(dbp, (t)val)) != 0)		\
 			goto nameerr;					\
 		continue;						\
 	}
@@ -678,6 +673,26 @@ err:	dbenv->err(dbenv, ret, "DB_ENV->open");
 		continue;						\
 	}
 
+/*
+ * The code to check a command-line or input header argument against a list
+ * of configuration options.  It's #defined because it's used in two places
+ * and the two places have gotten out of sync more than once.
+ */
+#define	CONFIGURATION_LIST_COMPARE					\
+	NUMBER(name, value, "bt_minkey", set_bt_minkey, u_int32_t);	\
+	  FLAG(name, value, "chksum", DB_CHKSUM);			\
+	NUMBER(name, value, "db_lorder", set_lorder, int);		\
+	NUMBER(name, value, "db_pagesize", set_pagesize, u_int32_t);	\
+	  FLAG(name, value, "duplicates", DB_DUP);			\
+	  FLAG(name, value, "dupsort", DB_DUPSORT);			\
+	NUMBER(name, value, "extentsize", set_q_extentsize, u_int32_t);	\
+	NUMBER(name, value, "h_ffactor", set_h_ffactor, u_int32_t);	\
+	NUMBER(name, value, "h_nelem", set_h_nelem, u_int32_t);		\
+	NUMBER(name, value, "re_len", set_re_len, u_int32_t);		\
+	STRING(name, value, "re_pad", set_re_pad);			\
+	  FLAG(name, value, "recnum", DB_RECNUM);			\
+	  FLAG(name, value, "renumber", DB_RENUMBER)
+
 /*
  * configure --
  *	Handle command-line configuration options.
@@ -724,21 +739,7 @@ configure(dbenv, dbp, clp, subdbp, keysp)
 			continue;
 		}
 
-#ifdef notyet
-		NUMBER(name, value, "bt_maxkey", set_bt_maxkey, u_int32_t);
-#endif
-		NUMBER(name, value, "bt_minkey", set_bt_minkey, u_int32_t);
-		NUMBER(name, value, "db_lorder", set_lorder, int);
-		NUMBER(name, value, "db_pagesize", set_pagesize, u_int32_t);
-		FLAG(name, value, "chksum", DB_CHKSUM);
-		FLAG(name, value, "duplicates", DB_DUP);
-		FLAG(name, value, "dupsort", DB_DUPSORT);
-		NUMBER(name, value, "h_ffactor", set_h_ffactor, u_int32_t);
-		NUMBER(name, value, "h_nelem", set_h_nelem, u_int32_t);
-		NUMBER(name, value, "re_len", set_re_len, u_int32_t);
-		STRING(name, value, "re_pad", set_re_pad);
-		FLAG(name, value, "recnum", DB_RECNUM);
-		FLAG(name, value, "renumber", DB_RENUMBER);
+		CONFIGURATION_LIST_COMPARE;
 
 		dbp->errx(dbp,
 		    "unknown command-line configuration keyword \"%s\"", name);
@@ -748,7 +749,7 @@ configure(dbenv, dbp, clp, subdbp, keysp)
 
 nameerr:
 	dbp->err(dbp, ret, "%s: %s=%s", G(progname), name, value);
-	return (1);
+err:	return (1);
 }
 
 /*
@@ -770,7 +771,7 @@ rheader(dbenv, dbp, dbtypep, subdbp, checkprintp, keysp)
 
 	*dbtypep = DB_UNKNOWN;
 	*checkprintp = 0;
-	name = p = NULL;
+	name = NULL;
 
 	/*
 	 * We start with a smallish buffer;  most headers are small.
@@ -779,10 +780,8 @@ rheader(dbenv, dbp, dbtypep, subdbp, checkprintp, keysp)
 	buflen = 4096;
 	if (G(hdrbuf) == NULL) {
 		hdr = 0;
-		if ((buf = malloc(buflen)) == NULL) {
-memerr:			dbp->errx(dbp, "could not allocate buffer %d", buflen);
-			return (1);
-		}
+		if ((buf = malloc(buflen)) == NULL)
+			goto memerr;
 		G(hdrbuf) = buf;
 		G(origline) = G(lineno);
 	} else {
@@ -831,7 +830,6 @@ memerr:			dbp->errx(dbp, "could not allocate buffer %d", buflen);
 		start += linelen;
 
 		if (name != NULL) {
-			*p = '=';
 			free(name);
 			name = NULL;
 		}
@@ -844,7 +842,27 @@ memerr:			dbp->errx(dbp, "could not allocate buffer %d", buflen);
 
 		value = p--;
 
-		if (name[0] == '\0' || value[0] == '\0')
+		if (name[0] == '\0')
+			goto badfmt;
+
+		/*
+		 * The only values that may be zero-length are database names.
+		 * In the original Berkeley DB code it was possible to create
+		 * zero-length database names, and the db_load code was then
+		 * changed to allow such databases to be be dumped and loaded.
+		 * [#8204]
+		 */
+		if (strcmp(name, "database") == 0 ||
+		    strcmp(name, "subdatabase") == 0) {
+			if ((ret = convprintable(dbenv, value, subdbp)) != 0) {
+				dbp->err(dbp, ret, "error reading db name");
+				goto err;
+			}
+			continue;
+		}
+
+		/* No other values may be zero-length. */
+		if (value[0] == '\0')
 			goto badfmt;
 
 		if (strcmp(name, "HEADER") == 0)
@@ -895,14 +913,6 @@ memerr:			dbp->errx(dbp, "could not allocate buffer %d", buflen);
 			dbp->errx(dbp, "line %lu: unknown type", G(lineno));
 			goto err;
 		}
-		if (strcmp(name, "database") == 0 ||
-		    strcmp(name, "subdatabase") == 0) {
-			if ((ret = convprintable(dbenv, value, subdbp)) != 0) {
-				dbp->err(dbp, ret, "error reading db name");
-				goto err;
-			}
-			continue;
-		}
 		if (strcmp(name, "keys") == 0) {
 			if (strcmp(value, "1") == 0)
 				*keysp = 1;
@@ -915,22 +925,7 @@ memerr:			dbp->errx(dbp, "could not allocate buffer %d", buflen);
 			continue;
 		}
 
-#ifdef notyet
-		NUMBER(name, value, "bt_maxkey", set_bt_maxkey, u_int32_t);
-#endif
-		NUMBER(name, value, "bt_minkey", set_bt_minkey, u_int32_t);
-		NUMBER(name, value, "db_lorder", set_lorder, int);
-		NUMBER(name, value, "db_pagesize", set_pagesize, u_int32_t);
-		NUMBER(name, value, "extentsize", set_q_extentsize, u_int32_t);
-		FLAG(name, value, "chksum", DB_CHKSUM);
-		FLAG(name, value, "duplicates", DB_DUP);
-		FLAG(name, value, "dupsort", DB_DUPSORT);
-		NUMBER(name, value, "h_ffactor", set_h_ffactor, u_int32_t);
-		NUMBER(name, value, "h_nelem", set_h_nelem, u_int32_t);
-		NUMBER(name, value, "re_len", set_re_len, u_int32_t);
-		STRING(name, value, "re_pad", set_re_pad);
-		FLAG(name, value, "recnum", DB_RECNUM);
-		FLAG(name, value, "renumber", DB_RENUMBER);
+		CONFIGURATION_LIST_COMPARE;
 
 		dbp->errx(dbp,
 		    "unknown input-file header configuration keyword \"%s\"",
@@ -938,22 +933,70 @@ memerr:			dbp->errx(dbp, "could not allocate buffer %d", buflen);
 		goto err;
 	}
 	ret = 0;
+
 	if (0) {
 nameerr:	dbp->err(dbp, ret, "%s: %s=%s", G(progname), name, value);
-err:		ret = 1;
+		ret = 1;
 	}
 	if (0) {
 badfmt:		dbp->errx(dbp, "line %lu: unexpected format", G(lineno));
 		ret = 1;
 	}
-	if (name != NULL) {
-		if (p != NULL)
-			*p = '=';
-		free(name);
+	if (0) {
+memerr:		dbp->errx(dbp, "unable to allocate memory");
+err:		ret = 1;
 	}
+	if (name != NULL)
+		free(name);
 	return (ret);
 }
 
+/*
+ * Macro to convert a pair of hex bytes to a decimal value.
+ *
+ * !!!
+ * Note that this macro is side-effect safe.  This was done deliberately,
+ * callers depend on it.
+ */
+#define	DIGITIZE(store, v1, v2) {					\
+	char _v1, _v2;							\
+	_v1 = (v1);							\
+	_v2 = (v2);							\
+	if ((_v1) > 'f' || (_v2) > 'f')					\
+		return (badend(dbenv));					\
+	(store) =							\
+	((_v1) == '0' ? 0 :						\
+	((_v1) == '1' ? 1 :						\
+	((_v1) == '2' ? 2 :						\
+	((_v1) == '3' ? 3 :						\
+	((_v1) == '4' ? 4 :						\
+	((_v1) == '5' ? 5 :						\
+	((_v1) == '6' ? 6 :						\
+	((_v1) == '7' ? 7 :						\
+	((_v1) == '8' ? 8 :						\
+	((_v1) == '9' ? 9 :						\
+	((_v1) == 'a' ? 10 :						\
+	((_v1) == 'b' ? 11 :						\
+	((_v1) == 'c' ? 12 :						\
+	((_v1) == 'd' ? 13 :						\
+	((_v1) == 'e' ? 14 : 15))))))))))))))) << 4 |			\
+	((_v2) == '0' ? 0 :						\
+	((_v2) == '1' ? 1 :						\
+	((_v2) == '2' ? 2 :						\
+	((_v2) == '3' ? 3 :						\
+	((_v2) == '4' ? 4 :						\
+	((_v2) == '5' ? 5 :						\
+	((_v2) == '6' ? 6 :						\
+	((_v2) == '7' ? 7 :						\
+	((_v2) == '8' ? 8 :						\
+	((_v2) == '9' ? 9 :						\
+	((_v2) == 'a' ? 10 :						\
+	((_v2) == 'b' ? 11 :						\
+	((_v2) == 'c' ? 12 :						\
+	((_v2) == 'd' ? 13 :						\
+	((_v2) == 'e' ? 14 : 15)))))))))))))));				\
+}
+
 /*
  * convprintable --
  *	Convert a printable-encoded string into a newly allocated string.
@@ -972,33 +1015,27 @@ convprintable(dbenv, instr, outstrp)
 	DB_ENV *dbenv;
 	char *instr, **outstrp;
 {
-	char c, *outstr;
-	int e1, e2;
+	char *outstr;
 
 	/*
 	 * Just malloc a string big enough for the whole input string;
 	 * the output string will be smaller (or of equal length).
+	 *
+	 * Note that we may be passed a zero-length string and need to
+	 * be able to duplicate it.
 	 */
 	if ((outstr = malloc(strlen(instr) + 1)) == NULL)
 		return (ENOMEM);
 
 	*outstrp = outstr;
 
-	e1 = e2 = 0;
 	for ( ; *instr != '\0'; instr++)
 		if (*instr == '\\') {
 			if (*++instr == '\\') {
 				*outstr++ = '\\';
 				continue;
 			}
-			c = digitize(dbenv, *instr, &e1) << 4;
-			c |= digitize(dbenv, *++instr, &e2);
-			if (e1 || e2) {
-				badend(dbenv);
-				return (EINVAL);
-			}
-
-			*outstr++ = c;
+			DIGITIZE(*outstr++, *instr, *++instr);
 		} else
 			*outstr++ = *instr;
 
@@ -1018,21 +1055,20 @@ dbt_rprint(dbenv, dbtp)
 {
 	u_int32_t len;
 	u_int8_t *p;
-	int c1, c2, e, escape, first;
+	int c1, c2, escape, first;
 	char buf[32];
 
 	++G(lineno);
 
 	first = 1;
-	e = escape = 0;
+	escape = 0;
 	for (p = dbtp->data, len = 0; (c1 = getchar()) != '\n';) {
 		if (c1 == EOF) {
 			if (len == 0) {
 				G(endofile) = G(endodata) = 1;
 				return (0);
 			}
-			badend(dbenv);
-			return (1);
+			return (badend(dbenv));
 		}
 		if (first) {
 			first = 0;
@@ -1041,10 +1077,8 @@ dbt_rprint(dbenv, dbtp)
 					buf[0] = c1;
 					if (fgets(buf + 1,
 					    sizeof(buf) - 1, stdin) == NULL ||
-					    strcmp(buf, "DATA=END\n") != 0) {
-						badend(dbenv);
-						return (1);
-					}
+					    strcmp(buf, "DATA=END\n") != 0)
+						return (badend(dbenv));
 					G(endodata) = 1;
 					return (0);
 				}
@@ -1053,14 +1087,9 @@ dbt_rprint(dbenv, dbtp)
 		}
 		if (escape) {
 			if (c1 != '\\') {
-				if ((c2 = getchar()) == EOF) {
-					badend(dbenv);
-					return (1);
-				}
-				c1 = digitize(dbenv,
-				    c1, &e) << 4 | digitize(dbenv, c2, &e);
-				if (e)
-					return (1);
+				if ((c2 = getchar()) == EOF)
+					return (badend(dbenv));
+				DIGITIZE(c1, c1, c2);
 			}
 			escape = 0;
 		} else
@@ -1096,21 +1125,19 @@ dbt_rdump(dbenv, dbtp)
 {
 	u_int32_t len;
 	u_int8_t *p;
-	int c1, c2, e, first;
+	int c1, c2, first;
 	char buf[32];
 
 	++G(lineno);
 
 	first = 1;
-	e = 0;
 	for (p = dbtp->data, len = 0; (c1 = getchar()) != '\n';) {
 		if (c1 == EOF) {
 			if (len == 0) {
 				G(endofile) = G(endodata) = 1;
 				return (0);
 			}
-			badend(dbenv);
-			return (1);
+			return (badend(dbenv));
 		}
 		if (first) {
 			first = 0;
@@ -1119,20 +1146,16 @@ dbt_rdump(dbenv, dbtp)
 					buf[0] = c1;
 					if (fgets(buf + 1,
 					    sizeof(buf) - 1, stdin) == NULL ||
-					    strcmp(buf, "DATA=END\n") != 0) {
-						badend(dbenv);
-						return (1);
-					}
+					    strcmp(buf, "DATA=END\n") != 0)
+						return (badend(dbenv));
 					G(endodata) = 1;
 					return (0);
 				}
 				continue;
 			}
 		}
-		if ((c2 = getchar()) == EOF) {
-			badend(dbenv);
-			return (1);
-		}
+		if ((c2 = getchar()) == EOF)
+			return (badend(dbenv));
 		if (len >= dbtp->ulen - 10) {
 			dbtp->ulen *= 2;
 			if ((dbtp->data =
@@ -1143,9 +1166,7 @@ dbt_rdump(dbenv, dbtp)
 			p = (u_int8_t *)dbtp->data + len;
 		}
 		++len;
-		*p++ = digitize(dbenv, c1, &e) << 4 | digitize(dbenv, c2, &e);
-		if (e)
-			return (1);
+		DIGITIZE(*p++, c1, c2);
 	}
 	dbtp->size = len;
 
@@ -1178,7 +1199,7 @@ dbt_rrecno(dbenv, dbtp, ishex)
 	}
 
 	if (buf[0] != ' ')
-		goto bad;
+		goto err;
 
 	/*
 	 * If we're expecting a hex key, do an in-place conversion
@@ -1194,22 +1215,22 @@ dbt_rrecno(dbenv, dbtp, ishex)
 			 * end-of-string conditions.
 			 */
 			if (*q++ != '3')
-				goto bad;
+				goto err;
 			if (*q == '\n' || *q == '\0')
-				goto bad;
+				goto err;
 			*p++ = *q++;
 		}
 		*p = '\0';
 	}
 
-	if (__db_getulong(dbenv, G(progname), buf + 1, 0, 0, &recno)) {
-bad:		badend(dbenv);
-		return (1);
-	}
+	if (__db_getulong(dbenv, G(progname), buf + 1, 0, 0, &recno))
+		goto err;
 
 	*((db_recno_t *)dbtp->data) = recno;
 	dbtp->size = sizeof(db_recno_t);
 	return (0);
+
+err:	return (badend(dbenv));
 }
 
 int
@@ -1226,42 +1247,6 @@ dbt_to_recno(dbenv, dbt, recnop)
 	return (__db_getulong(dbenv, G(progname), buf, 0, 0, (u_long *)recnop));
 }
 
-/*
- * digitize --
- *	Convert a character to an integer.
- */
-int
-digitize(dbenv, c, errorp)
-	DB_ENV *dbenv;
-	int c, *errorp;
-{
-	switch (c) {			/* Don't depend on ASCII ordering. */
-	case '0': return (0);
-	case '1': return (1);
-	case '2': return (2);
-	case '3': return (3);
-	case '4': return (4);
-	case '5': return (5);
-	case '6': return (6);
-	case '7': return (7);
-	case '8': return (8);
-	case '9': return (9);
-	case 'a': return (10);
-	case 'b': return (11);
-	case 'c': return (12);
-	case 'd': return (13);
-	case 'e': return (14);
-	case 'f': return (15);
-	default:			/* Not possible. */
-		break;
-	}
-
-	dbenv->errx(dbenv, "unexpected hexadecimal value");
-	*errorp = 1;
-
-	return (0);
-}
-
 /*
  * badnum --
  *	Display the bad number message.
@@ -1278,11 +1263,12 @@ badnum(dbenv)
  * badend --
  *	Display the bad end to input message.
  */
-void
+int
 badend(dbenv)
 	DB_ENV *dbenv;
 {
 	dbenv->errx(dbenv, "unexpected end of input data or key/data pair");
+	return (1);
 }
 
 /*
@@ -1292,17 +1278,16 @@ badend(dbenv)
 int
 usage()
 {
-	(void)fprintf(stderr, "%s\n\t%s\n",
-	    "usage: db_load [-nTV] [-c name=value] [-f file]",
+	(void)fprintf(stderr, "usage: %s %s\n\t%s\n", progname,
+	    "[-nTV] [-c name=value] [-f file]",
     "[-h home] [-P password] [-t btree | hash | recno | queue] db_file");
-	(void)fprintf(stderr, "%s\n",
-	    "usage: db_load -r lsn | fileid [-h home] [-P password] db_file");
+	(void)fprintf(stderr, "usage: %s %s\n",
+	    progname, "-r lsn | fileid [-h home] [-P password] db_file");
 	return (EXIT_FAILURE);
 }
 
 int
-version_check(progname)
-	const char *progname;
+version_check()
 {
 	int v_major, v_minor, v_patch;
 
