@@ -1,15 +1,18 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2005
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 1996,2007 Oracle.  All rights reserved.
  *
- * $Id: lock.h,v 12.7 2005/10/07 20:21:23 ubell Exp $
+ * $Id: lock.h,v 12.18 2007/05/17 18:46:15 bostic Exp $
  */
 
 #ifndef	_DB_LOCK_H_
 #define	_DB_LOCK_H_
 
+#if defined(__cplusplus)
+extern "C" {
+#endif
+
 #define	DB_LOCK_DEFAULT_N	1000	/* Default # of locks in region. */
 
 /*
@@ -37,30 +40,26 @@
 	    (m) == DB_LOCK_IWRITE || (m) == DB_LOCK_IWR)
 
 /*
- * Lock timers.
+ * Macros to lock/unlock the lock region as a whole.
+ * IF we are using multiple mutexes in the lock region these are a noop.
  */
-typedef struct {
-	u_int32_t	tv_sec;		/* Seconds. */
-	u_int32_t	tv_usec;	/* Microseconds. */
-} db_timeval_t;
-
-#define	LOCK_TIME_ISVALID(time)		((time)->tv_sec != 0)
-#define	LOCK_SET_TIME_INVALID(time)	((time)->tv_sec = 0)
-#define	LOCK_TIME_ISMAX(time)		((time)->tv_sec == UINT32_MAX)
-#define	LOCK_SET_TIME_MAX(time)		((time)->tv_sec = UINT32_MAX)
-#define	LOCK_TIME_EQUAL(t1, t2)						\
-	((t1)->tv_sec == (t2)->tv_sec && (t1)->tv_usec == (t2)->tv_usec)
-#define	LOCK_TIME_GREATER(t1, t2)					\
-	((t1)->tv_sec > (t2)->tv_sec ||					\
-	((t1)->tv_sec == (t2)->tv_sec && (t1)->tv_usec > (t2)->tv_usec))
-
-/* Macros to lock/unlock the lock region as a whole. */
+#ifdef HAVE_FINE_GRAINED_LOCK_MANAGER
+#define	LOCK_SYSTEM_LOCK(dbenv)
+#define	LOCK_SYSTEM_UNLOCK(dbenv)
+#else
 #define	LOCK_SYSTEM_LOCK(dbenv)						\
-	MUTEX_LOCK(dbenv, ((DB_LOCKREGION *)((DB_LOCKTAB *)		\
-	    (dbenv)->lk_handle)->reginfo.primary)->mtx_region)
+	MUTEX_LOCK(dbenv, ((DB_LOCKREGION *)				\
+	    (dbenv)->lk_handle->reginfo.primary)->mtx_region)
 #define	LOCK_SYSTEM_UNLOCK(dbenv)					\
-	MUTEX_UNLOCK(dbenv, ((DB_LOCKREGION *)((DB_LOCKTAB *)		\
-	    (dbenv)->lk_handle)->reginfo.primary)->mtx_region)
+	MUTEX_UNLOCK(dbenv, ((DB_LOCKREGION *)				\
+	    (dbenv)->lk_handle->reginfo.primary)->mtx_region)
+#endif
+#define	LOCK_REGION_LOCK(dbenv)						\
+	MUTEX_LOCK(dbenv, ((DB_LOCKREGION *)				\
+	    (dbenv)->lk_handle->reginfo.primary)->mtx_region)
+#define	LOCK_REGION_UNLOCK(dbenv)					\
+	MUTEX_UNLOCK(dbenv, ((DB_LOCKREGION *)				\
+	    (dbenv)->lk_handle->reginfo.primary)->mtx_region)
 
 /*
  * DB_LOCKREGION --
@@ -71,14 +70,19 @@ typedef struct __db_lockregion {
 
 	u_int32_t	need_dd;	/* flag for deadlock detector */
 	u_int32_t	detect;		/* run dd on every conflict */
-	db_timeval_t	next_timeout;	/* next time to expire a lock */
+	db_timespec	next_timeout;	/* next time to expire a lock */
+#ifdef HAVE_FINE_GRAINED_LOCK_MANAGER
+	db_mutex_t	mtx_locks;	/* mutex for lock allocation. */
+	db_mutex_t	mtx_objs;	/* mutex for lock object allocation. */
+	db_mutex_t	mtx_lockers;	/* mutex for locker allocation. */
+#endif
 					/* free lock header */
 	SH_TAILQ_HEAD(__flock) free_locks;
 					/* free obj header */
 	SH_TAILQ_HEAD(__fobj) free_objs;
+	SH_TAILQ_HEAD(__dobj) dd_objs;	/* objects with waiters */
 					/* free locker header */
 	SH_TAILQ_HEAD(__flocker) free_lockers;
-	SH_TAILQ_HEAD(__dobj) dd_objs;	/* objects with waiters */
 	SH_TAILQ_HEAD(__lkrs) lockers;	/* list of lockers */
 
 	db_timeout_t	lk_timeout;	/* timeout for locks. */
@@ -89,6 +93,10 @@ typedef struct __db_lockregion {
 
 	roff_t		conf_off;	/* offset of conflicts array */
 	roff_t		obj_off;	/* offset of object hash table */
+#ifdef HAVE_FINE_GRAINED_LOCK_MANAGER
+	roff_t		mtx_off;	/* offset of object mutex table */
+#endif
+	roff_t		stat_off;	/* offset to object hash stats */
 	roff_t		locker_off;	/* offset of locker hash table */
 
 	DB_LOCK_STAT	stat;		/* stats about locking. */
@@ -109,6 +117,8 @@ typedef struct __sh_dbt {
  * Object structures;  these live in the object hash table.
  */
 typedef struct __db_lockobj {
+	u_int32_t	indx;		/* Hash index of this object. */
+	u_int32_t	generation;	/* Generation of this object. */
 	SH_DBT	lockobj;		/* Identifies object locked. */
 	SH_TAILQ_ENTRY links;		/* Links for free list or hash list. */
 	SH_TAILQ_ENTRY dd_links;	/* Links for dd list. */
@@ -124,7 +134,7 @@ typedef struct __db_lockobj {
 /*
  * Locker structures; these live in the locker hash table.
  */
-typedef struct __db_locker {
+struct __db_locker {
 	u_int32_t id;			/* Locker id. */
 
 	pid_t pid;			/* Process owning locker ID */
@@ -146,8 +156,8 @@ typedef struct __db_locker {
 	SH_TAILQ_ENTRY links;		/* Links for free and hash list. */
 	SH_TAILQ_ENTRY ulinks;		/* Links in-use list. */
 	SH_LIST_HEAD(_held) heldby;	/* Locks held by this locker. */
-	db_timeval_t	lk_expire;	/* When current lock expires. */
-	db_timeval_t	tx_expire;	/* When this txn expires. */
+	db_timespec	lk_expire;	/* When current lock expires. */
+	db_timespec	tx_expire;	/* When this txn expires. */
 	db_timeout_t	lk_timeout;	/* How long do we let locks live. */
 
 #define	DB_LOCKER_DELETED	0x0001
@@ -155,20 +165,24 @@ typedef struct __db_locker {
 #define	DB_LOCKER_INABORT	0x0004
 #define	DB_LOCKER_TIMEOUT	0x0008
 	u_int32_t flags;
-} DB_LOCKER;
+};
 
 /*
  * DB_LOCKTAB --
  *	The primary library lock data structure (i.e., the one referenced
  * by the environment, as opposed to the internal one laid out in the region.)
  */
-typedef struct __db_locktab {
+struct __db_locktab {
 	DB_ENV		*dbenv;		/* Environment. */
 	REGINFO		 reginfo;	/* Region information. */
 	u_int8_t	*conflicts;	/* Pointer to conflict matrix. */
 	DB_HASHTAB	*obj_tab;	/* Beginning of object hash table. */
+#ifdef HAVE_FINE_GRAINED_LOCK_MANAGER
+	db_mutex_t	*obj_mtx;	/* Object mutex array. */
+#endif
+	DB_LOCK_HSTAT	*obj_stat;	/* Object hash stats array. */
 	DB_HASHTAB	*locker_tab;	/* Beginning of locker hash table. */
-} DB_LOCKTAB;
+};
 
 /*
  * Test for conflicts.
@@ -187,13 +201,14 @@ struct __db_lock {
 	 */
 	db_mutex_t	mtx_lock;
 
-	u_int32_t	holder;		/* Who holds this lock. */
+	roff_t		holder;		/* Who holds this lock. */
 	u_int32_t	gen;		/* Generation count. */
 	SH_TAILQ_ENTRY	links;		/* Free or holder/waiter list. */
 	SH_LIST_ENTRY	locker_links;	/* List of locks held by a locker. */
 	u_int32_t	refcount;	/* Reference count the lock. */
 	db_lockmode_t	mode;		/* What sort of lock. */
 	roff_t		obj;		/* Relative offset of object struct. */
+	u_int32_t	indx;		/* Hash index of this object. */
 	db_status_t	status;		/* Status of this lock. */
 };
 
@@ -211,16 +226,59 @@ struct __db_lock {
 #define	DB_LOCK_FREE		0x040000
 #define	DB_LOCK_NOPROMOTE	0x080000
 #define	DB_LOCK_UNLINK		0x100000
-#define	DB_LOCK_NOREGION	0x200000
 #define	DB_LOCK_NOWAITERS	0x400000
 
 /*
  * Macros to get/release different types of mutexes.
  */
+#ifdef HAVE_FINE_GRAINED_LOCK_MANAGER
+/*
+ * All operations on a lock object are preceded by getting a lock on
+ * the mutex for its hash bucket.  Lock structures associated with
+ * an object are protected by this mutex as well.  If holding a hash bucket
+ * we may call LOCK_OBJECTS to allocate a new object.
+ */
+#define	OBJECT_LOCK(lt, reg, obj, ndx) do {				\
+	ndx = __lock_ohash(obj) % (reg)->object_t_size;			\
+	MUTEX_LOCK((lt)->dbenv, (lt)->obj_mtx[ndx]);			\
+} while (0)
+#define	OBJECT_LOCK_NDX(lt, ndx)					\
+	MUTEX_LOCK((lt)->dbenv, (lt)->obj_mtx[ndx])
+#define	OBJECT_UNLOCK(lt, ndx)						\
+	MUTEX_UNLOCK((lt)->dbenv, (lt)->obj_mtx[ndx])
+
+/*
+ * These mutexes protect the allocation/deallocation and the queues
+ * of the lock objects, lock structures and locks.
+ * It is assumed that all accesses to a locker are single threaded
+ * since transactions are single threaded.  The exception is in the
+ * deadlock detector where we look at the last lock held by the locker
+ * with some care.
+ */
+#define	LOCK_OBJECTS(dbenv, region)					\
+	MUTEX_LOCK(dbenv, (region)->mtx_objs)
+#define	UNLOCK_OBJECTS(dbenv, region)					\
+	MUTEX_UNLOCK(dbenv, (region)->mtx_objs)
+#define	LOCK_LOCKS(dbenv, region)					\
+	MUTEX_LOCK(dbenv, (region)->mtx_locks)
+#define	UNLOCK_LOCKS(dbenv, region)					\
+	MUTEX_UNLOCK(dbenv, (region)->mtx_locks)
+#define	LOCK_LOCKERS(dbenv, region)					\
+	MUTEX_LOCK(dbenv, (region)->mtx_lockers)
+#define	UNLOCK_LOCKERS(dbenv, region)					\
+	MUTEX_UNLOCK(dbenv, (region)->mtx_lockers)
+#else
 #define	OBJECT_LOCK(lt, reg, obj, ndx)					\
 	ndx = __lock_ohash(obj) % (reg)->object_t_size
-#define	SHOBJECT_LOCK(lt, reg, shobj, ndx)				\
-	ndx = __lock_lhash(shobj) % (reg)->object_t_size
+#define	OBJECT_LOCK_NDX(lt, ndx)
+#define	OBJECT_UNLOCK(lt, ndx)
+#define	LOCK_OBJECTS(dbenv, region)
+#define	UNLOCK_OBJECTS(dbenv, region)
+#define	LOCK_LOCKS(dbenv, region)
+#define	UNLOCK_LOCKS(dbenv, region)
+#define	LOCK_LOCKERS(dbenv, region)
+#define	UNLOCK_LOCKERS(dbenv, region)
+#endif
 
 /*
  * __lock_locker_hash --
@@ -229,8 +287,12 @@ struct __db_lock {
  *	just return the locker value.
  */
 #define	__lock_locker_hash(locker)	(locker)
-#define	LOCKER_LOCK(lt, reg, locker, ndx)				\
+#define	LOCKER_HASH(lt, reg, locker, ndx)				\
 	ndx = __lock_locker_hash(locker) % (reg)->locker_t_size;
 
+#if defined(__cplusplus)
+}
+#endif
+
 #include "dbinc_auto/lock_ext.h"
 #endif /* !_DB_LOCK_H_ */
