@@ -1,30 +1,19 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1999, 2000
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 1999,2007 Oracle.  All rights reserved.
  *
- * $Id: hash_verify.c,v 1.20.2.1 2000/06/30 14:02:11 krinsky Exp $
+ * $Id: hash_verify.c,v 12.26 2007/07/02 16:58:02 alexg Exp $
  */
 
 #include "db_config.h"
 
-#ifndef lint
-static const char revid[] = "$Id: hash_verify.c,v 1.20.2.1 2000/06/30 14:02:11 krinsky Exp $";
-#endif /* not lint */
-
-#ifndef NO_SYSTEM_INCLUDES
-#include <sys/types.h>
-
-#include <errno.h>
-#include <string.h>
-#endif
-
 #include "db_int.h"
-#include "db_page.h"
-#include "db_verify.h"
-#include "btree.h"
-#include "hash.h"
+#include "dbinc/db_page.h"
+#include "dbinc/db_verify.h"
+#include "dbinc/btree.h"
+#include "dbinc/hash.h"
+#include "dbinc/mp.h"
 
 static int __ham_dups_unsorted __P((DB *, u_int8_t *, u_int32_t));
 static int __ham_vrfy_bucket __P((DB *, VRFY_DBINFO *, HMETA *, u_int32_t,
@@ -51,15 +40,18 @@ __ham_vrfy_meta(dbp, vdp, m, pgno, flags)
 	db_pgno_t pgno;
 	u_int32_t flags;
 {
+	DB_ENV *dbenv;
 	HASH *hashp;
 	VRFY_PAGEINFO *pip;
 	int i, ret, t_ret, isbad;
 	u_int32_t pwr, mbucket;
-	u_int32_t (*hfunc) __P((const void *, u_int32_t));
+	u_int32_t (*hfunc) __P((DB *, const void *, u_int32_t));
+
+	dbenv = dbp->dbenv;
+	isbad = 0;
 
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		return (ret);
-	isbad = 0;
 
 	hashp = dbp->h_internal;
 
@@ -82,14 +74,14 @@ __ham_vrfy_meta(dbp, vdp, m, pgno, flags)
 
 	/* h_charkey */
 	if (!LF_ISSET(DB_NOORDERCHK))
-		if (m->h_charkey != hfunc(CHARKEY, sizeof(CHARKEY))) {
-			EPRINT((dbp->dbenv,
-"Database has different custom hash function; reverify with DB_NOORDERCHK set"
-			    ));
+		if (m->h_charkey != hfunc(dbp, CHARKEY, sizeof(CHARKEY))) {
+			EPRINT((dbenv,
+"Page %lu: database has custom hash function; reverify with DB_NOORDERCHK set",
+			    (u_long)pgno));
 			/*
-			 * Return immediately;  this is probably a sign
-			 * of user error rather than database corruption, so
-			 * we want to avoid extraneous errors.
+			 * Return immediately;  this is probably a sign of user
+			 * error rather than database corruption, so we want to
+			 * avoid extraneous errors.
 			 */
 			isbad = 1;
 			goto err;
@@ -97,9 +89,9 @@ __ham_vrfy_meta(dbp, vdp, m, pgno, flags)
 
 	/* max_bucket must be less than the last pgno. */
 	if (m->max_bucket > vdp->last_pgno) {
-		EPRINT((dbp->dbenv,
-		    "Impossible max_bucket %lu on meta page %lu",
-		    m->max_bucket, pgno));
+		EPRINT((dbenv,
+		    "Page %lu: Impossible max_bucket %lu on meta page",
+		    (u_long)pgno, (u_long)m->max_bucket));
 		/*
 		 * Most other fields depend somehow on max_bucket, so
 		 * we just return--there will be lots of extraneous
@@ -113,21 +105,19 @@ __ham_vrfy_meta(dbp, vdp, m, pgno, flags)
 	 * max_bucket, high_mask and low_mask: high_mask must be one
 	 * less than the next power of two above max_bucket, and
 	 * low_mask must be one less than the power of two below it.
-	 *
-	 *
 	 */
 	pwr = (m->max_bucket == 0) ? 1 : 1 << __db_log2(m->max_bucket + 1);
 	if (m->high_mask != pwr - 1) {
-		EPRINT((dbp->dbenv,
-		    "Incorrect high_mask %lu on page %lu, should be %lu",
-		    m->high_mask, pgno, pwr - 1));
+		EPRINT((dbenv,
+		    "Page %lu: incorrect high_mask %lu, should be %lu",
+		    (u_long)pgno, (u_long)m->high_mask, (u_long)pwr - 1));
 		isbad = 1;
 	}
 	pwr >>= 1;
 	if (m->low_mask != pwr - 1) {
-		EPRINT((dbp->dbenv,
-		    "Incorrect low_mask %lu on page %lu, should be %lu",
-		    m->low_mask, pgno, pwr - 1));
+		EPRINT((dbenv,
+		    "Page %lu: incorrect low_mask %lu, should be %lu",
+		    (u_long)pgno, (u_long)m->low_mask, (u_long)pwr - 1));
 		isbad = 1;
 	}
 
@@ -140,9 +130,9 @@ __ham_vrfy_meta(dbp, vdp, m, pgno, flags)
 	 * which could make nelem go "negative".
 	 */
 	if (m->nelem > 0x80000000) {
-		EPRINT((dbp->dbenv,
-		    "Suspiciously high nelem of %lu on page %lu",
-		    m->nelem, pgno));
+		EPRINT((dbenv,
+		    "Page %lu: suspiciously high nelem of %lu",
+		    (u_long)pgno, (u_long)m->nelem));
 		isbad = 1;
 		pip->h_nelem = 0;
 	} else
@@ -163,16 +153,18 @@ __ham_vrfy_meta(dbp, vdp, m, pgno, flags)
 		 * than last_pgno.
 		 */
 		mbucket = (1 << i) - 1;
-		if (mbucket + m->spares[__db_log2(mbucket + 1)] >
-		    vdp->last_pgno) {
-			EPRINT((dbp->dbenv,
-			    "Spares array entry %lu, page %lu is invalid",
-			    i, pgno));
+		if (BS_TO_PAGE(mbucket, m->spares) > vdp->last_pgno) {
+			EPRINT((dbenv,
+			    "Page %lu: spares array entry %d is invalid",
+			    (u_long)pgno, i));
 			isbad = 1;
 		}
 	}
 
-err:	if ((t_ret = __db_vrfy_putpageinfo(vdp, pip)) != 0 && ret == 0)
+err:	if ((t_ret = __db_vrfy_putpageinfo(dbenv, vdp, pip)) != 0 && ret == 0)
+		ret = t_ret;
+	if (LF_ISSET(DB_SALVAGE) &&
+	   (t_ret = __db_salvage_markdone(vdp, pgno)) != 0 && ret == 0)
 		ret = t_ret;
 	return ((ret == 0 && isbad == 1) ? DB_VERIFY_BAD : ret);
 }
@@ -192,23 +184,20 @@ __ham_vrfy(dbp, vdp, h, pgno, flags)
 	db_pgno_t pgno;
 	u_int32_t flags;
 {
+	DB_ENV *dbenv;
 	VRFY_PAGEINFO *pip;
 	u_int32_t ent, himark, inpend;
+	db_indx_t *inp;
 	int isbad, ret, t_ret;
 
+	dbenv = dbp->dbenv;
 	isbad = 0;
+
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		return (ret);
 
-	/* Sanity check our flags and page type. */
-	if ((ret = __db_fchk(dbp->dbenv, "__ham_vrfy",
-	    flags, DB_AGGRESSIVE | DB_NOORDERCHK | DB_SALVAGE)) != 0)
-		goto err;
-
-	if (TYPE(h) != P_HASH) {
-		TYPE_ERR_PRINT(dbp->dbenv, "__ham_vrfy", pgno, TYPE(h));
-		DB_ASSERT(0);
-		ret = EINVAL;
+	if (TYPE(h) != P_HASH && TYPE(h) != P_HASH_UNSORTED) {
+		ret = __db_unknown_path(dbenv, "__ham_vrfy");
 		goto err;
 	}
 
@@ -228,31 +217,37 @@ __ham_vrfy(dbp, vdp, h, pgno, flags)
 	 * In any case, we return immediately if things are bad, as it would
 	 * be unsafe to proceed.
 	 */
+	inp = P_INP(dbp, h);
 	for (ent = 0, himark = dbp->pgsize,
-	    inpend = (u_int8_t *)h->inp - (u_int8_t *)h;
+	    inpend = (u_int32_t)((u_int8_t *)inp - (u_int8_t *)h);
 	    ent < NUM_ENT(h); ent++)
-		if (h->inp[ent] >= himark) {
-			EPRINT((dbp->dbenv,
-			    "Item %lu on page %lu out of order or nonsensical",
-			    ent, pgno));
+		if (inp[ent] >= himark) {
+			EPRINT((dbenv,
+			    "Page %lu: item %lu is out of order or nonsensical",
+			    (u_long)pgno, (u_long)ent));
 			isbad = 1;
 			goto err;
 		} else if (inpend >= himark) {
-			EPRINT((dbp->dbenv,
-			    "inp array collided with data on page %lu",
-			    pgno));
+			EPRINT((dbenv,
+			    "Page %lu: entries array collided with data",
+			    (u_long)pgno));
 			isbad = 1;
 			goto err;
 
 		} else {
-			himark = h->inp[ent];
+			himark = inp[ent];
 			inpend += sizeof(db_indx_t);
 			if ((ret = __ham_vrfy_item(
 			    dbp, vdp, pgno, h, ent, flags)) != 0)
 				goto err;
 		}
 
-err:	if ((t_ret = __db_vrfy_putpageinfo(vdp, pip)) != 0 && ret == 0)
+	if (!LF_ISSET(DB_NOORDERCHK) && TYPE(h) == P_HASH &&
+	    (ret = __ham_verify_sorted_page(dbp, NULL, h)) != 0)
+		isbad = 1;
+
+err:	if ((t_ret =
+	    __db_vrfy_putpageinfo(dbenv, vdp, pip)) != 0 && ret == 0)
 		ret = t_ret;
 	return (ret == 0 && isbad == 1 ? DB_VERIFY_BAD : ret);
 }
@@ -281,7 +276,7 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		return (ret);
 
-	switch (HPAGE_TYPE(h, i)) {
+	switch (HPAGE_TYPE(dbp, h, i)) {
 	case H_KEYDATA:
 		/* Nothing to do here--everything but the type field is data */
 		break;
@@ -289,8 +284,8 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 		/* Are we a datum or a key?  Better be the former. */
 		if (i % 2 == 0) {
 			EPRINT((dbp->dbenv,
-			    "Hash key stored as duplicate at page %lu item %lu",
-			    pip->pgno, i));
+			    "Page %lu: hash key stored as duplicate item %lu",
+			    (u_long)pip->pgno, (u_long)i));
 		}
 		/*
 		 * Dups are encoded as a series within a single HKEYDATA,
@@ -302,16 +297,16 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 		 * Note that at this point, we've verified item i-1, so
 		 * it's safe to use LEN_HKEYDATA (which looks at inp[i-1]).
 		 */
-		len = LEN_HKEYDATA(h, dbp->pgsize, i);
-		databuf = HKEYDATA_DATA(P_ENTRY(h, i));
+		len = LEN_HKEYDATA(dbp, h, dbp->pgsize, i);
+		databuf = HKEYDATA_DATA(P_ENTRY(dbp, h, i));
 		for (offset = 0; offset < len; offset += DUP_SIZE(dlen)) {
 			memcpy(&dlen, databuf + offset, sizeof(db_indx_t));
 
 			/* Make sure the length is plausible. */
 			if (offset + DUP_SIZE(dlen) > len) {
 				EPRINT((dbp->dbenv,
-			    "Duplicate item %lu, page %lu has bad length",
-				    i, pip->pgno));
+			    "Page %lu: duplicate item %lu has bad length",
+				    (u_long)pip->pgno, (u_long)i));
 				ret = DB_VERIFY_BAD;
 				goto err;
 			}
@@ -325,8 +320,8 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 			    sizeof(db_indx_t));
 			if (elen != dlen) {
 				EPRINT((dbp->dbenv,
-		"Duplicate item %lu, page %lu has two different lengths",
-				    i, pip->pgno));
+		"Page %lu: duplicate item %lu has two different lengths",
+				    (u_long)pip->pgno, (u_long)i));
 				ret = DB_VERIFY_BAD;
 				goto err;
 			}
@@ -338,12 +333,12 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 		break;
 	case H_OFFPAGE:
 		/* Offpage item.  Make sure pgno is sane, save off. */
-		memcpy(&hop, P_ENTRY(h, i), HOFFPAGE_SIZE);
+		memcpy(&hop, P_ENTRY(dbp, h, i), HOFFPAGE_SIZE);
 		if (!IS_VALID_PGNO(hop.pgno) || hop.pgno == pip->pgno ||
 		    hop.pgno == PGNO_INVALID) {
 			EPRINT((dbp->dbenv,
-			    "Offpage item %lu, page %lu has bad page number",
-			    i, pip->pgno));
+			    "Page %lu: offpage item %lu has bad pgno %lu",
+			    (u_long)pip->pgno, (u_long)i, (u_long)hop.pgno));
 			ret = DB_VERIFY_BAD;
 			goto err;
 		}
@@ -356,12 +351,12 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 		break;
 	case H_OFFDUP:
 		/* Offpage duplicate item.  Same drill. */
-		memcpy(&hod, P_ENTRY(h, i), HOFFDUP_SIZE);
+		memcpy(&hod, P_ENTRY(dbp, h, i), HOFFDUP_SIZE);
 		if (!IS_VALID_PGNO(hod.pgno) || hod.pgno == pip->pgno ||
 		    hod.pgno == PGNO_INVALID) {
 			EPRINT((dbp->dbenv,
-			    "Offpage item %lu, page %lu has bad page number",
-			    i, pip->pgno));
+			    "Page %lu: offpage item %lu has bad page number",
+			    (u_long)pip->pgno, (u_long)i));
 			ret = DB_VERIFY_BAD;
 			goto err;
 		}
@@ -374,12 +369,14 @@ __ham_vrfy_item(dbp, vdp, pgno, h, i, flags)
 		break;
 	default:
 		EPRINT((dbp->dbenv,
-		    "Item %i, page %lu has bad type", i, pip->pgno));
+		    "Page %lu: item %lu has bad type",
+		    (u_long)pip->pgno, (u_long)i));
 		ret = DB_VERIFY_BAD;
 		break;
 	}
 
-err:	if ((t_ret = __db_vrfy_putpageinfo(vdp, pip)) != 0 && ret == 0)
+err:	if ((t_ret =
+	    __db_vrfy_putpageinfo(dbp->dbenv, vdp, pip)) != 0 && ret == 0)
 		ret = t_ret;
 	return (ret);
 }
@@ -399,29 +396,32 @@ __ham_vrfy_structure(dbp, vdp, meta_pgno, flags)
 	u_int32_t flags;
 {
 	DB *pgset;
+	DB_MPOOLFILE *mpf;
 	HMETA *m;
 	PAGE *h;
 	VRFY_PAGEINFO *pip;
 	int isbad, p, ret, t_ret;
 	db_pgno_t pgno;
-	u_int32_t bucket;
+	u_int32_t bucket, spares_entry;
 
-	ret = isbad = 0;
-	h = NULL;
+	mpf = dbp->mpf;
 	pgset = vdp->pgset;
+	h = NULL;
+	ret = isbad = 0;
 
 	if ((ret = __db_vrfy_pgset_get(pgset, meta_pgno, &p)) != 0)
 		return (ret);
 	if (p != 0) {
 		EPRINT((dbp->dbenv,
-		    "Hash meta page %lu referenced twice", meta_pgno));
+		    "Page %lu: Hash meta page referenced twice",
+		    (u_long)meta_pgno));
 		return (DB_VERIFY_BAD);
 	}
 	if ((ret = __db_vrfy_pgset_inc(pgset, meta_pgno)) != 0)
 		return (ret);
 
 	/* Get the meta page;  we'll need it frequently. */
-	if ((ret = memp_fget(dbp->mpf, &meta_pgno, 0, &m)) != 0)
+	if ((ret = __memp_fget(mpf, &meta_pgno, NULL, 0, &m)) != 0)
 		return (ret);
 
 	/* Loop through bucket by bucket. */
@@ -436,55 +436,71 @@ __ham_vrfy_structure(dbp, vdp, meta_pgno, flags)
 
 	/*
 	 * There may be unused hash pages corresponding to buckets
-	 * that are part of the current doubling but which are higher
-	 * than max_bucket.  Loop through them, too, and make sure they're
-	 * all empty.
+	 * that have been allocated but not yet used.  These may be
+	 * part of the current doubling above max_bucket, or they may
+	 * correspond to buckets that were used in a transaction
+	 * that then aborted.
+	 *
+	 * Loop through them, as far as the spares array defines them,
+	 * and make sure they're all empty.
+	 *
+	 * Note that this should be safe, since we've already verified
+	 * that the spares array is sane.
 	 */
-	for (bucket = m->max_bucket + 1; bucket <= m->high_mask; bucket++) {
-		pgno = bucket + m->spares[__db_log2(bucket + 1)];
+	for (bucket = m->max_bucket + 1; spares_entry = __db_log2(bucket + 1),
+	    spares_entry < NCACHED && m->spares[spares_entry] != 0; bucket++) {
+		pgno = BS_TO_PAGE(bucket, m->spares);
 		if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 			goto err;
 
 		/* It's okay if these pages are totally zeroed;  unmark it. */
 		F_CLR(pip, VRFY_IS_ALLZEROES);
 
-		if (pip->type != P_HASH) {
+		/* It's also OK if this page is simply invalid. */
+		if (pip->type == P_INVALID) {
+			if ((ret = __db_vrfy_putpageinfo(dbp->dbenv,
+			    vdp, pip)) != 0)
+				goto err;
+			continue;
+		}
+
+		if (pip->type != P_HASH && pip->type != P_HASH_UNSORTED) {
 			EPRINT((dbp->dbenv,
-			    "Hash bucket %lu maps to non-hash page %lu",
-			    bucket, pgno));
+			    "Page %lu: hash bucket %lu maps to non-hash page",
+			    (u_long)pgno, (u_long)bucket));
 			isbad = 1;
 		} else if (pip->entries != 0) {
 			EPRINT((dbp->dbenv,
-			    "Non-empty page %lu in unused hash bucket %lu",
-			    pgno, bucket));
+		    "Page %lu: non-empty page in unused hash bucket %lu",
+			    (u_long)pgno, (u_long)bucket));
 			isbad = 1;
 		} else {
 			if ((ret = __db_vrfy_pgset_get(pgset, pgno, &p)) != 0)
 				goto err;
 			if (p != 0) {
 				EPRINT((dbp->dbenv,
-				    "Hash page %lu above max_bucket referenced",
-				    pgno));
+				    "Page %lu: above max_bucket referenced",
+				    (u_long)pgno));
 				isbad = 1;
 			} else {
 				if ((ret =
 				    __db_vrfy_pgset_inc(pgset, pgno)) != 0)
 					goto err;
-				if ((ret =
-				    __db_vrfy_putpageinfo(vdp, pip)) != 0)
+				if ((ret = __db_vrfy_putpageinfo(dbp->dbenv,
+				    vdp, pip)) != 0)
 					goto err;
 				continue;
 			}
 		}
 
 		/* If we got here, it's an error. */
-		(void)__db_vrfy_putpageinfo(vdp, pip);
+		(void)__db_vrfy_putpageinfo(dbp->dbenv, vdp, pip);
 		goto err;
 	}
 
-err:	if ((t_ret = memp_fput(dbp->mpf, m, 0)) != 0)
+err:	if ((t_ret = __memp_fput(mpf, m, dbp->priority)) != 0)
 		return (t_ret);
-	if (h != NULL && (t_ret = memp_fput(dbp->mpf, h, 0)) != 0)
+	if (h != NULL && (t_ret = __memp_fput(mpf, h, dbp->priority)) != 0)
 		return (t_ret);
 	return ((isbad == 1 && ret == 0) ? DB_VERIFY_BAD: ret);
 }
@@ -500,14 +516,16 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 	HMETA *m;
 	u_int32_t bucket, flags;
 {
+	DB_ENV *dbenv;
 	HASH *hashp;
 	VRFY_CHILDINFO *child;
 	VRFY_PAGEINFO *mip, *pip;
 	int ret, t_ret, isbad, p;
 	db_pgno_t pgno, next_pgno;
 	DBC *cc;
-	u_int32_t (*hfunc) __P((const void *, u_int32_t));
+	u_int32_t (*hfunc) __P((DB *, const void *, u_int32_t));
 
+	dbenv = dbp->dbenv;
 	isbad = 0;
 	pip = NULL;
 	cc = NULL;
@@ -522,23 +540,26 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 		return (ret);
 
 	/* Calculate the first pgno for this bucket. */
-	pgno = bucket + m->spares[__db_log2(bucket + 1)];
+	pgno = BS_TO_PAGE(bucket, m->spares);
 
 	if ((ret = __db_vrfy_getpageinfo(vdp, pgno, &pip)) != 0)
 		goto err;
 
 	/* Make sure we got a plausible page number. */
-	if (pgno > vdp->last_pgno || pip->type != P_HASH) {
-		EPRINT((dbp->dbenv, "Bucket %lu has impossible first page %lu",
-		    bucket, pgno));
+	if (pgno > vdp->last_pgno || 
+	    (pip->type != P_HASH && pip->type != P_HASH_UNSORTED)) {
+		EPRINT((dbenv,
+		    "Page %lu: impossible first page in bucket %lu",
+		    (u_long)pgno, (u_long)bucket));
 		/* Unsafe to continue. */
 		isbad = 1;
 		goto err;
 	}
 
 	if (pip->prev_pgno != PGNO_INVALID) {
-		EPRINT((dbp->dbenv,
-		    "First hash page %lu in bucket %lu has a prev_pgno", pgno));
+		EPRINT((dbenv,
+		    "Page %lu: first page in hash bucket %lu has a prev_pgno",
+		    (u_long)pgno, (u_long)bucket));
 		isbad = 1;
 	}
 
@@ -550,11 +571,16 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 
 	/* Loop until we find a fatal bug, or until we run out of pages. */
 	for (;;) {
+		/* Provide feedback on our progress to the application. */
+		if (!LF_ISSET(DB_SALVAGE))
+			__db_vrfy_struct_feedback(dbp, vdp);
+
 		if ((ret = __db_vrfy_pgset_get(vdp->pgset, pgno, &p)) != 0)
 			goto err;
 		if (p != 0) {
-			EPRINT((dbp->dbenv,
-			    "Hash page %lu referenced twice", pgno));
+			EPRINT((dbenv,
+			    "Page %lu: hash page referenced twice",
+			    (u_long)pgno));
 			isbad = 1;
 			/* Unsafe to continue. */
 			goto err;
@@ -574,11 +600,11 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 		F_CLR(pip, VRFY_IS_ALLZEROES);
 
 		/* If we have dups, our meta page had better know about it. */
-		if (F_ISSET(pip, VRFY_HAS_DUPS)
-		    && !F_ISSET(mip, VRFY_HAS_DUPS)) {
-			EPRINT((dbp->dbenv,
-		    "Duplicates present in non-duplicate database, page %lu",
-			    pgno));
+		if (F_ISSET(pip, VRFY_HAS_DUPS) &&
+		    !F_ISSET(mip, VRFY_HAS_DUPS)) {
+			EPRINT((dbenv,
+		    "Page %lu: duplicates present in non-duplicate database",
+			    (u_long)pgno));
 			isbad = 1;
 		}
 
@@ -588,9 +614,9 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 		 */
 		if (F_ISSET(mip, VRFY_HAS_DUPSORT) &&
 		    F_ISSET(pip, VRFY_DUPS_UNSORTED)) {
-			EPRINT((dbp->dbenv,
-			    "Unsorted dups in sorted-dup database, page %lu",
-			    pgno));
+			EPRINT((dbenv,
+			    "Page %lu: unsorted dups in sorted-dup database",
+			    (u_long)pgno));
 			isbad = 1;
 		}
 
@@ -601,7 +627,8 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 		    ret = __db_vrfy_ccnext(cc, &child))
 			if (child->type == V_OVERFLOW) {
 				if ((ret = __db_vrfy_ovfl_structure(dbp, vdp,
-				    child->pgno, child->tlen, flags)) != 0) {
+				    child->pgno, child->tlen,
+				    flags | ST_OVFL_LEAF)) != 0) {
 					if (ret == DB_VERIFY_BAD)
 						isbad = 1;
 					else
@@ -615,8 +642,8 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 				}
 				if ((ret = __bam_vrfy_subtree(dbp, vdp,
 				    child->pgno, NULL, NULL,
-				    flags | ST_RECNUM, NULL,
-				    NULL, NULL)) != 0) {
+				    flags | ST_RECNUM | ST_DUPSET | ST_TOPLEVEL,
+				    NULL, NULL, NULL)) != 0) {
 					if (ret == DB_VERIFY_BAD)
 						isbad = 1;
 					else
@@ -638,7 +665,7 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 		}
 
 		next_pgno = pip->next_pgno;
-		ret = __db_vrfy_putpageinfo(vdp, pip);
+		ret = __db_vrfy_putpageinfo(dbenv, vdp, pip);
 
 		pip = NULL;
 		if (ret != 0)
@@ -649,9 +676,9 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 
 		/* We already checked this, but just in case... */
 		if (!IS_VALID_PGNO(next_pgno)) {
-			DB_ASSERT(0);
-			EPRINT((dbp->dbenv,
-			    "Hash page %lu has bad next_pgno", pgno));
+			EPRINT((dbenv,
+			    "Page %lu: hash page has bad next_pgno",
+			    (u_long)pgno));
 			isbad = 1;
 			goto err;
 		}
@@ -660,8 +687,9 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 			goto err;
 
 		if (pip->prev_pgno != pgno) {
-			EPRINT((dbp->dbenv, "Hash page %lu has bad prev_pgno",
-			    next_pgno));
+			EPRINT((dbenv,
+			    "Page %lu: hash page has bad prev_pgno",
+			    (u_long)next_pgno));
 			isbad = 1;
 		}
 		pgno = next_pgno;
@@ -669,11 +697,11 @@ __ham_vrfy_bucket(dbp, vdp, m, bucket, flags)
 
 err:	if (cc != NULL && ((t_ret = __db_vrfy_ccclose(cc)) != 0) && ret == 0)
 		ret = t_ret;
-	if (mip != NULL && ((t_ret = __db_vrfy_putpageinfo(vdp, mip)) != 0) &&
-	    ret == 0)
+	if (mip != NULL && ((t_ret =
+	    __db_vrfy_putpageinfo(dbenv, vdp, mip)) != 0) && ret == 0)
 		ret = t_ret;
-	if (pip != NULL && ((t_ret = __db_vrfy_putpageinfo(vdp, pip)) != 0) &&
-	    ret == 0)
+	if (pip != NULL && ((t_ret =
+	    __db_vrfy_putpageinfo(dbenv, vdp, pip)) != 0) && ret == 0)
 		ret = t_ret;
 	return ((ret == 0 && isbad == 1) ? DB_VERIFY_BAD : ret);
 }
@@ -684,7 +712,7 @@ err:	if (cc != NULL && ((t_ret = __db_vrfy_ccclose(cc)) != 0) && ret == 0)
  *
  * PUBLIC: int __ham_vrfy_hashing __P((DB *,
  * PUBLIC:     u_int32_t, HMETA *, u_int32_t, db_pgno_t, u_int32_t,
- * PUBLIC:     u_int32_t (*) __P((const void *, u_int32_t))));
+ * PUBLIC:     u_int32_t (*) __P((DB *, const void *, u_int32_t))));
  */
 int
 __ham_vrfy_hashing(dbp, nentries, m, thisbucket, pgno, flags, hfunc)
@@ -694,19 +722,22 @@ __ham_vrfy_hashing(dbp, nentries, m, thisbucket, pgno, flags, hfunc)
 	u_int32_t thisbucket;
 	db_pgno_t pgno;
 	u_int32_t flags;
-	u_int32_t (*hfunc) __P((const void *, u_int32_t));
+	u_int32_t (*hfunc) __P((DB *, const void *, u_int32_t));
 {
 	DBT dbt;
+	DB_MPOOLFILE *mpf;
 	PAGE *h;
 	db_indx_t i;
 	int ret, t_ret, isbad;
 	u_int32_t hval, bucket;
 
+	mpf = dbp->mpf;
 	ret = isbad = 0;
+
 	memset(&dbt, 0, sizeof(DBT));
 	F_SET(&dbt, DB_DBT_REALLOC);
 
-	if ((ret = memp_fget(dbp->mpf, &pgno, 0, &h)) != 0)
+	if ((ret = __memp_fget(mpf, &pgno, NULL, 0, &h)) != 0)
 		return (ret);
 
 	for (i = 0; i < nentries; i += 2) {
@@ -718,9 +749,9 @@ __ham_vrfy_hashing(dbp, nentries, m, thisbucket, pgno, flags, hfunc)
 		 * can tweak this a bit if this proves to be a bottleneck,
 		 * but for now, take the easy route.
 		 */
-		if ((ret = __db_ret(dbp, h, i, &dbt, NULL, NULL)) != 0)
+		if ((ret = __db_ret(dbp, NULL, h, i, &dbt, NULL, NULL)) != 0)
 			goto err;
-		hval = hfunc(dbt.data, dbt.size);
+		hval = hfunc(dbp, dbt.data, dbt.size);
 
 		bucket = hval & m->high_mask;
 		if (bucket > m->max_bucket)
@@ -728,15 +759,15 @@ __ham_vrfy_hashing(dbp, nentries, m, thisbucket, pgno, flags, hfunc)
 
 		if (bucket != thisbucket) {
 			EPRINT((dbp->dbenv,
-			    "Item %lu on page %lu hashes incorrectly",
-			    i, pgno));
+			    "Page %lu: item %lu hashes incorrectly",
+			    (u_long)pgno, (u_long)i));
 			isbad = 1;
 		}
 	}
 
 err:	if (dbt.data != NULL)
-		__os_free(dbt.data, 0);
-	if ((t_ret = memp_fput(dbp->mpf, h, 0)) != 0)
+		__os_ufree(dbp->dbenv, dbt.data);
+	if ((t_ret = __memp_fput(mpf, h, dbp->priority)) != 0)
 		return (t_ret);
 
 	return ((ret == 0 && isbad == 1) ? DB_VERIFY_BAD : ret);
@@ -763,17 +794,15 @@ __ham_salvage(dbp, vdp, pgno, h, handle, callback, flags)
 	DBT dbt, unkdbt;
 	db_pgno_t dpgno;
 	int ret, err_ret, t_ret;
-	u_int32_t himark, tlen;
-	u_int8_t *hk;
+	u_int32_t himark, i;
+	u_int8_t *hk, *p;
 	void *buf;
-	u_int32_t dlen, len, i;
+	db_indx_t dlen, len, tlen;
 
 	memset(&dbt, 0, sizeof(DBT));
 	dbt.flags = DB_DBT_REALLOC;
 
-	memset(&unkdbt, 0, sizeof(DBT));
-	unkdbt.size = strlen("UNKNOWN") + 1;
-	unkdbt.data = "UNKNOWN";
+	DB_INIT_DBT(unkdbt, "UNKNOWN", sizeof("UNKNOWN") - 1);
 
 	err_ret = 0;
 
@@ -781,7 +810,7 @@ __ham_salvage(dbp, vdp, pgno, h, handle, callback, flags)
 	 * Allocate a buffer for overflow items.  Start at one page;
 	 * __db_safe_goff will realloc as needed.
 	 */
-	if ((ret = __os_malloc(dbp->dbenv, dbp->pgsize, NULL, &buf)) != 0)
+	if ((ret = __os_malloc(dbp->dbenv, dbp->pgsize, &buf)) != 0)
 		return (ret);
 
 	himark = dbp->pgsize;
@@ -798,33 +827,30 @@ __ham_salvage(dbp, vdp, pgno, h, handle, callback, flags)
 			break;
 
 		if (ret == 0) {
-			hk = P_ENTRY(h, i);
-			len = LEN_HKEYDATA(h, dbp->pgsize, i);
-			if ((u_int32_t)(hk + len - (u_int8_t *)h) >
+			/* Set len to total entry length. */
+			len = LEN_HITEM(dbp, h, dbp->pgsize, i);
+			hk = P_ENTRY(dbp, h, i);
+			if (len == 0 || len > dbp->pgsize ||
+			    (u_int32_t)(hk + len - (u_int8_t *)h) >
 			    dbp->pgsize) {
-				/*
-				 * Item is unsafely large;  either continue
-				 * or set it to the whole page, depending on
-				 * aggressiveness.
-				 */
-				if (!LF_ISSET(DB_AGGRESSIVE))
-					continue;
-				len = dbp->pgsize -
-				    (u_int32_t)(hk - (u_int8_t *)h);
+				/* Item is unsafely large; skip it. */
 				err_ret = DB_VERIFY_BAD;
+				continue;
 			}
 			switch (HPAGE_PTYPE(hk)) {
 			default:
 				if (!LF_ISSET(DB_AGGRESSIVE))
 					break;
 				err_ret = DB_VERIFY_BAD;
-				/* FALLTHROUGH */
+				break;
 			case H_KEYDATA:
+				/* Update len to size of item. */
+				len = LEN_HKEYDATA(dbp, h, dbp->pgsize, i);
 keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
 				dbt.size = len;
 				dbt.data = buf;
-				if ((ret = __db_prdbt(&dbt,
-				    0, " ", handle, callback, 0, NULL)) != 0)
+				if ((ret = __db_vrfy_prdbt(&dbt,
+				    0, " ", handle, callback, 0, vdp)) != 0)
 					err_ret = ret;
 				break;
 			case H_OFFPAGE:
@@ -837,25 +863,26 @@ keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
 				if ((ret = __db_safe_goff(dbp, vdp,
 				    dpgno, &dbt, &buf, flags)) != 0) {
 					err_ret = ret;
-					(void)__db_prdbt(&unkdbt, 0, " ",
-					    handle, callback, 0, NULL);
+					(void)__db_vrfy_prdbt(&unkdbt, 0, " ",
+					    handle, callback, 0, vdp);
 					break;
 				}
-				if ((ret = __db_prdbt(&dbt,
-				    0, " ", handle, callback, 0, NULL)) != 0)
+				if ((ret = __db_vrfy_prdbt(&dbt,
+				    0, " ", handle, callback, 0, vdp)) != 0)
 					err_ret = ret;
 				break;
 			case H_OFFDUP:
-				if (len < HOFFPAGE_SIZE) {
+				if (len < HOFFDUP_SIZE) {
 					err_ret = DB_VERIFY_BAD;
 					continue;
 				}
 				memcpy(&dpgno,
-				    HOFFPAGE_PGNO(hk), sizeof(dpgno));
+				    HOFFDUP_PGNO(hk), sizeof(dpgno));
 				/* UNKNOWN iff pgno is bad or we're a key. */
 				if (!IS_VALID_PGNO(dpgno) || (i % 2 == 0)) {
-					if ((ret = __db_prdbt(&unkdbt, 0, " ",
-					    handle, callback, 0, NULL)) != 0)
+					if ((ret =
+					    __db_vrfy_prdbt(&unkdbt, 0, " ",
+					    handle, callback, 0, vdp)) != 0)
 						err_ret = ret;
 				} else if ((ret = __db_salvage_duptree(dbp,
 				    vdp, dpgno, &dbt, handle, callback,
@@ -863,6 +890,7 @@ keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
 					err_ret = ret;
 				break;
 			case H_DUPLICATE:
+				len = LEN_HKEYDATA(dbp, h, dbp->pgsize, i);
 				/*
 				 * We're a key;  printing dups will seriously
 				 * foul the output.  If we're being aggressive,
@@ -876,7 +904,12 @@ keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
 					break;
 				}
 
-				/* Too small to have any data. */
+				/*
+				 * Check if too small to have any data.
+				 * But first, we have to update the len to
+				 * reflect the size of the data not the
+				 * size of the on-page entry.
+				 */
 				if (len <
 				    HKEYDATA_SIZE(2 * sizeof(db_indx_t))) {
 					err_ret = DB_VERIFY_BAD;
@@ -886,19 +919,21 @@ keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
 				/* Loop until we hit the total length. */
 				for (tlen = 0; tlen + sizeof(db_indx_t) < len;
 				    tlen += dlen) {
+					p = HKEYDATA_DATA(hk) + tlen;
 					tlen += sizeof(db_indx_t);
-					memcpy(&dlen, hk, sizeof(db_indx_t));
+					memcpy(&dlen, p, sizeof(db_indx_t));
+					p += sizeof(db_indx_t);
 					/*
 					 * If dlen is too long, print all the
 					 * rest of the dup set in a chunk.
 					 */
 					if (dlen + tlen > len)
 						dlen = len - tlen;
-					memcpy(buf, hk + tlen, dlen);
+					memcpy(buf, p, dlen);
 					dbt.size = dlen;
 					dbt.data = buf;
-					if ((ret = __db_prdbt(&dbt, 0, " ",
-					    handle, callback, 0, NULL)) != 0)
+					if ((ret = __db_vrfy_prdbt(&dbt, 0, " ",
+					    handle, callback, 0, vdp)) != 0)
 						err_ret = ret;
 					tlen += sizeof(db_indx_t);
 				}
@@ -907,6 +942,7 @@ keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
 		}
 	}
 
+	__os_free(dbp->dbenv, buf);
 	if ((t_ret = __db_salvage_markdone(vdp, pgno)) != 0)
 		return (t_ret);
 	return ((ret == 0 && err_ret != 0) ? err_ret : ret);
@@ -920,13 +956,15 @@ keydata:			memcpy(buf, HKEYDATA_DATA(hk), len);
  * PUBLIC: int __ham_meta2pgset __P((DB *, VRFY_DBINFO *, HMETA *, u_int32_t,
  * PUBLIC:     DB *));
  */
-int __ham_meta2pgset(dbp, vdp, hmeta, flags, pgset)
+int
+__ham_meta2pgset(dbp, vdp, hmeta, flags, pgset)
 	DB *dbp;
 	VRFY_DBINFO *vdp;
 	HMETA *hmeta;
 	u_int32_t flags;
 	DB *pgset;
 {
+	DB_MPOOLFILE *mpf;
 	PAGE *h;
 	db_pgno_t pgno;
 	u_int32_t bucket, totpgs;
@@ -938,8 +976,9 @@ int __ham_meta2pgset(dbp, vdp, hmeta, flags, pgset)
 	 */
 	COMPQUIET(flags, 0);
 
-	DB_ASSERT(pgset != NULL);
+	DB_ASSERT(dbp->dbenv, pgset != NULL);
 
+	mpf = dbp->mpf;
 	totpgs = 0;
 
 	/*
@@ -947,7 +986,7 @@ int __ham_meta2pgset(dbp, vdp, hmeta, flags, pgset)
 	 * page(s) for each one.
 	 */
 	for (bucket = 0; bucket <= hmeta->max_bucket; bucket++) {
-		pgno = bucket + hmeta->spares[__db_log2(bucket + 1)];
+		pgno = BS_TO_PAGE(bucket, hmeta->spares);
 
 		/*
 		 * We know the initial pgno is safe because the spares array has
@@ -956,33 +995,37 @@ int __ham_meta2pgset(dbp, vdp, hmeta, flags, pgset)
 		 * Safely walk the list of pages in this bucket.
 		 */
 		for (;;) {
-			if ((ret = memp_fget(dbp->mpf, &pgno, 0, &h)) != 0)
+			if ((ret = __memp_fget(mpf, &pgno, NULL, 0, &h)) != 0)
 				return (ret);
-			if (TYPE(h) == P_HASH) {
+			if (TYPE(h) == P_HASH || TYPE(h) == P_HASH_UNSORTED) {
 
 				/*
 				 * Make sure we don't go past the end of
 				 * pgset.
 				 */
 				if (++totpgs > vdp->last_pgno) {
-					(void)memp_fput(dbp->mpf, h, 0);
+					(void)__memp_fput(mpf,
+					    h, dbp->priority);
 					return (DB_VERIFY_BAD);
 				}
 				if ((ret =
-				    __db_vrfy_pgset_inc(pgset, pgno)) != 0)
+				    __db_vrfy_pgset_inc(pgset, pgno)) != 0) {
+					(void)__memp_fput(mpf,
+					    h, dbp->priority);
 					return (ret);
+				}
 
 				pgno = NEXT_PGNO(h);
 			} else
 				pgno = PGNO_INVALID;
 
-			if ((ret = memp_fput(dbp->mpf, h, 0)) != 0)
+			if ((ret = __memp_fput(mpf, h, dbp->priority)) != 0)
 				return (ret);
 
 			/* If the new pgno is wonky, go onto the next bucket. */
 			if (!IS_VALID_PGNO(pgno) ||
 			    pgno == PGNO_INVALID)
-				goto nextbucket;
+				break;
 
 			/*
 			 * If we've touched this page before, we have a cycle;
@@ -991,9 +1034,8 @@ int __ham_meta2pgset(dbp, vdp, hmeta, flags, pgset)
 			if ((ret = __db_vrfy_pgset_get(pgset, pgno, &val)) != 0)
 				return (ret);
 			if (val != 0)
-				goto nextbucket;
+				break;
 		}
-nextbucket:	;
 	}
 	return (0);
 }
@@ -1012,7 +1054,7 @@ __ham_dups_unsorted(dbp, buf, len)
 {
 	DBT a, b;
 	db_indx_t offset, dlen;
-	int (*func) __P((const DBT *, const DBT *));
+	int (*func) __P((DB *, const DBT *, const DBT *));
 
 	memset(&a, 0, sizeof(DBT));
 	memset(&b, 0, sizeof(DBT));
@@ -1029,7 +1071,7 @@ __ham_dups_unsorted(dbp, buf, len)
 		b.data = buf + offset + sizeof(db_indx_t);
 		b.size = dlen;
 
-		if (a.data != NULL && func(&a, &b) > 0)
+		if (a.data != NULL && func(dbp, &a, &b) > 0)
 			return (1);
 
 		a.data = b.data;
