@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1996-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1996,2007 Oracle.  All rights reserved.
  *
- * $Id: mp_fput.c,v 12.22 2006/09/07 20:05:33 bostic Exp $
+ * $Id: mp_fput.c,v 12.36 2007/06/05 11:55:28 mjc Exp $
  */
 
 #include "db_config.h"
@@ -19,12 +18,14 @@ static int __memp_reset_lru __P((DB_ENV *, REGINFO *));
  * __memp_fput_pp --
  *	DB_MPOOLFILE->put pre/post processing.
  *
- * PUBLIC: int __memp_fput_pp __P((DB_MPOOLFILE *, void *, u_int32_t));
+ * PUBLIC: int __memp_fput_pp
+ * PUBLIC:     __P((DB_MPOOLFILE *, void *, DB_CACHE_PRIORITY, u_int32_t));
  */
 int
-__memp_fput_pp(dbmfp, pgaddr, flags)
+__memp_fput_pp(dbmfp, pgaddr, priority, flags)
 	DB_MPOOLFILE *dbmfp;
 	void *pgaddr;
+	DB_CACHE_PRIORITY priority;
 	u_int32_t flags;
 {
 	DB_ENV *dbenv;
@@ -33,10 +34,14 @@ __memp_fput_pp(dbmfp, pgaddr, flags)
 
 	dbenv = dbmfp->dbenv;
 	PANIC_CHECK(dbenv);
+	if (flags != 0)
+		return (__db_ferr(dbenv, "DB_MPOOLFILE->put", 0));
+
+	MPF_ILLEGAL_BEFORE_OPEN(dbmfp, "DB_MPOOLFILE->put");
 
 	ENV_ENTER(dbenv, ip);
 
-	ret = __memp_fput(dbmfp, pgaddr, flags);
+	ret = __memp_fput(dbmfp, pgaddr, priority);
 	if (IS_ENV_REPLICATED(dbenv) &&
 	    (t_ret = __op_rep_exit(dbenv)) != 0 && ret == 0)
 		ret = t_ret;
@@ -49,46 +54,29 @@ __memp_fput_pp(dbmfp, pgaddr, flags)
  * __memp_fput --
  *	DB_MPOOLFILE->put.
  *
- * PUBLIC: int __memp_fput __P((DB_MPOOLFILE *, void *, u_int32_t));
+ * PUBLIC: int __memp_fput __P((DB_MPOOLFILE *, void *, DB_CACHE_PRIORITY));
  */
 int
-__memp_fput(dbmfp, pgaddr, flags)
+__memp_fput(dbmfp, pgaddr, priority)
 	DB_MPOOLFILE *dbmfp;
 	void *pgaddr;
-	u_int32_t flags;
+	DB_CACHE_PRIORITY priority;
 {
+	BH *bhp;
 	DB_ENV *dbenv;
 	DB_MPOOL *dbmp;
 	DB_MPOOL_HASH *hp;
 	MPOOL *c_mp;
 	MPOOLFILE *mfp;
-	BH *bhp;
-	u_int32_t n_cache;
-	int adjust, ret, t_ret;
+	REGINFO *infop;
+	int adjust, pfactor, ret, t_ret;
 
 	dbenv = dbmfp->dbenv;
-	MPF_ILLEGAL_BEFORE_OPEN(dbmfp, "DB_MPOOLFILE->put");
 	dbmp = dbenv->mp_handle;
 	mfp = dbmfp->mfp;
 	bhp = (BH *)((u_int8_t *)pgaddr - SSZA(BH, buf));
 	ret = 0;
 
-	/*
-	 * Check arguments, but don't fail because we want to unpin the page
-	 * regardless.  The problem is when running with replication.  There
-	 * is a reference count we incremented when __memp_fget was called,
-	 * and we need to unpin the page and decrement that reference count.
-	 * If we see flag problems, mark the page dirty.
-	 */
-	if (flags) {
-		if (__db_fchk(dbenv, "memp_fput", flags,
-		    DB_MPOOL_DISCARD) != 0) {
-			flags = 0;
-			ret = EINVAL;
-			DB_ASSERT(dbenv, 0);
-		}
-	}
-
 	/*
 	 * If we're mapping the file, there's nothing to do.  Because we can
 	 * stop mapping the file at any time, we have to check on each buffer
@@ -116,15 +104,10 @@ __memp_fput(dbmfp, pgaddr, flags)
 #endif
 
 	/* Convert a page address to a buffer header and hash bucket. */
-	n_cache = NCACHE(dbmp->reginfo[0].primary, bhp->mf_offset, bhp->pgno);
-	c_mp = dbmp->reginfo[n_cache].primary;
-	hp = R_ADDR(&dbmp->reginfo[n_cache], c_mp->htab);
-	hp = &hp[NBUCKET(c_mp, bhp->mf_offset, bhp->pgno)];
-
-	MUTEX_LOCK(dbenv, hp->mtx_hash);
-
-	if (LF_ISSET(DB_MPOOL_DISCARD))
-		F_SET(bhp, BH_DISCARD);
+	MP_GET_BUCKET(dbmfp, bhp->pgno, &infop, hp, ret);
+	if (ret != 0)
+		return (ret);
+	c_mp = infop->primary;
 
 	/*
 	 * Check for a reference count going to zero.  This can happen if the
@@ -141,14 +124,13 @@ __memp_fput(dbmfp, pgaddr, flags)
 	/* Note the activity so allocation won't decide to quit. */
 	++c_mp->put_counter;
 
-	/*
-	 * Mark the file dirty.  Check for a dirty bit on the buffer as well
-	 * as the dirty flag because the buffer might have been marked dirty
-	 * in the DB_MPOOLFILE->set method.
-	 */
-	if (F_ISSET(bhp, BH_DIRTY))
+	/* Mark the file dirty. */
+	if (F_ISSET(bhp, BH_DIRTY)) {
 		mfp->file_written = 1;
 
+		DB_ASSERT(dbenv, !SH_CHAIN_HASNEXT(bhp, vc));
+	}
+
 	/*
 	 * If more than one reference to the page or a reference other than a
 	 * thread waiting to flush the buffer to disk, we're done.  Ignore the
@@ -163,7 +145,8 @@ __memp_fput(dbmfp, pgaddr, flags)
 	MVCC_MPROTECT(bhp->buf, mfp->stat.st_pagesize, 0);
 
 	/* Update priority values. */
-	if (F_ISSET(bhp, BH_DISCARD) || mfp->priority == MPOOL_PRI_VERY_LOW)
+	if (priority == DB_PRIORITY_VERY_LOW ||
+	    mfp->priority == MPOOL_PRI_VERY_LOW)
 		bhp->priority = 0;
 	else {
 		/*
@@ -173,9 +156,31 @@ __memp_fput(dbmfp, pgaddr, flags)
 		 */
 		bhp->priority = c_mp->lru_count;
 
+		switch (priority) {
+		default:
+		case DB_PRIORITY_UNCHANGED:
+			pfactor = mfp->priority;
+			break;
+		case DB_PRIORITY_VERY_LOW:
+			pfactor = MPOOL_PRI_VERY_LOW;
+			break;
+		case DB_PRIORITY_LOW:
+			pfactor = MPOOL_PRI_LOW;
+			break;
+		case DB_PRIORITY_DEFAULT:
+			pfactor = MPOOL_PRI_DEFAULT;
+			break;
+		case DB_PRIORITY_HIGH:
+			pfactor = MPOOL_PRI_HIGH;
+			break;
+		case DB_PRIORITY_VERY_HIGH:
+			pfactor = MPOOL_PRI_VERY_HIGH;
+			break;
+		}
+
 		adjust = 0;
-		if (mfp->priority != 0)
-			adjust = (int)c_mp->stat.st_pages / mfp->priority;
+		if (pfactor != 0)
+			adjust = (int)c_mp->stat.st_pages / pfactor;
 
 		if (F_ISSET(bhp, BH_DIRTY))
 			adjust += (int)c_mp->stat.st_pages / MPOOL_PRI_DIRTY;
@@ -234,10 +239,9 @@ __memp_reset_lru(dbenv, infop)
 	BH *bhp, *tbhp;
 	DB_MPOOL_HASH *hp;
 	MPOOL *c_mp;
-	u_int32_t bucket;
+	u_int32_t bucket, priority;
 
 	c_mp = infop->primary;
-
 	/*
 	 * Update the counter so all future allocations will start at the
 	 * bottom.
@@ -253,19 +257,42 @@ __memp_reset_lru(dbenv, infop)
 		 * We can check for empty buckets before locking as we
 		 * only care if the pointer is zero or non-zero.
 		 */
-		if (SH_TAILQ_FIRST(&hp->hash_bucket, __bh) == NULL)
+		if (SH_TAILQ_FIRST(&hp->hash_bucket, __bh) == NULL) {
+			c_mp->lru_reset++;
 			continue;
+		}
 
 		MUTEX_LOCK(dbenv, hp->mtx_hash);
-		SH_TAILQ_FOREACH(bhp, &hp->hash_bucket, hq, __bh)
+		c_mp->lru_reset++;
+		/*
+		 * We need to take a little care that the bucket does
+		 * not become unsorted.  This is highly unlikely but
+		 * possible.
+		 */
+		priority = 0;
+		SH_TAILQ_FOREACH(bhp, &hp->hash_bucket, hq, __bh) {
 			for (tbhp = bhp; tbhp != NULL;
 			    tbhp = SH_CHAIN_PREV(tbhp, vc, __bh)) {
 				if (tbhp->priority != UINT32_MAX &&
-				    tbhp->priority > MPOOL_BASE_DECREMENT)
+				    tbhp->priority > MPOOL_BASE_DECREMENT) {
 					tbhp->priority -= MPOOL_BASE_DECREMENT;
+					if (tbhp->priority < priority)
+						tbhp->priority = priority;
+				}
 			}
+			priority = bhp->priority;
+		}
+		/*
+		 * Reset the hash bucket's priority.  The chain is never empty
+		 * in this case, so tbhp will never be NULL.
+		 */
+		if ((tbhp =
+		    SH_TAILQ_FIRST(&hp->hash_bucket, __bh)) != NULL)
+			hp->hash_priority = BH_PRIORITY(tbhp);
 		MUTEX_UNLOCK(dbenv, hp->mtx_hash);
 	}
+	c_mp->lru_reset = 0;
 
+	COMPQUIET(dbenv, NULL);
 	return (0);
 }
