@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2004-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 2004,2007 Oracle.  All rights reserved.
  *
- * $Id: os_truncate.c,v 12.7 2006/08/24 14:46:18 bostic Exp $
+ * $Id: os_truncate.c,v 12.10 2007/05/17 15:15:46 bostic Exp $
  */
 
 #include "db_config.h"
@@ -33,6 +32,11 @@ __os_truncate(dbenv, fhp, pgno, pgsize)
 	 */
 	offset = (off_t)pgsize * pgno;
 
+	if (dbenv != NULL &&
+	    FLD_ISSET(dbenv->verbose, DB_VERB_FILEOPS | DB_VERB_FILEOPS_ALL))
+		__db_msg(dbenv,
+		    "fileops: truncate %s to %lu", fhp->name, (u_long)offset);
+
 	if (DB_GLOBAL(j_ftruncate) != NULL)
 		ret = DB_GLOBAL(j_ftruncate)(fhp->fd, offset);
 	else {
