@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1997,2007 Oracle.  All rights reserved.
  *
- * $Id: os_unlink.c,v 12.8 2006/08/24 14:46:18 bostic Exp $
+ * $Id: os_unlink.c,v 12.11 2007/05/17 15:15:46 bostic Exp $
  */
 
 #include "db_config.h"
@@ -25,8 +24,13 @@ __os_region_unlink(dbenv, path)
 #ifdef HAVE_QNX
 	int ret;
 	char *newname;
+#endif
+	if (dbenv != NULL &&
+	    FLD_ISSET(dbenv->verbose, DB_VERB_FILEOPS | DB_VERB_FILEOPS_ALL))
+		__db_msg(dbenv, "fileops: unlink %s", path);
 
-	if ((ret = __os_shmname(dbenv, path, &newname)) != 0)
+#ifdef HAVE_QNX
+	if ((ret = __os_qnx_shmname(dbenv, path, &newname)) != 0)
 		goto err;
 
 	if ((ret = shm_unlink(newname)) != 0) {
@@ -59,6 +63,10 @@ __os_unlink(dbenv, path)
 {
 	int ret, t_ret;
 
+	if (dbenv != NULL &&
+	    FLD_ISSET(dbenv->verbose, DB_VERB_FILEOPS | DB_VERB_FILEOPS_ALL))
+		__db_msg(dbenv, "fileops: unlink %s", path);
+
 	if (DB_GLOBAL(j_unlink) != NULL)
 		ret = DB_GLOBAL(j_unlink)(path);
 	else
