@@ -1,10 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 1997,2007 Oracle.  All rights reserved.
  *
- * $Id: os_rename.c,v 12.6 2006/08/24 14:46:22 bostic Exp $
+ * $Id: os_rename.c,v 12.10 2007/05/17 15:15:49 bostic Exp $
  */
 
 #include "db_config.h"
@@ -24,6 +23,10 @@ __os_rename(dbenv, oldname, newname, silent)
 	_TCHAR *toldname, *tnewname;
 	int ret;
 
+	if (dbenv != NULL &&
+	    FLD_ISSET(dbenv->verbose, DB_VERB_FILEOPS | DB_VERB_FILEOPS_ALL))
+		__db_msg(dbenv, "fileops: rename %s to %s", oldname, newname);
+
 	TO_TSTRING(dbenv, oldname, toldname, ret);
 	if (ret != 0)
 		return (ret);
@@ -38,13 +41,16 @@ __os_rename(dbenv, oldname, newname, silent)
 
 	if (__os_posix_err(ret) == EEXIST) {
 		ret = 0;
+#ifndef DB_WINCE
 		if (__os_is_winnt()) {
 			if (!MoveFileEx(
 			    toldname, tnewname, MOVEFILE_REPLACE_EXISTING))
 				ret = __os_get_syserr();
-		} else {
+		} else
+#endif
+		{
 			/*
-			 * There is no MoveFileEx for Win9x/Me, so we have to
+			 * There is no MoveFileEx for Win9x/Me/CE, so we have to
 			 * do the best we can.  Note that the MoveFile call
 			 * above would have succeeded if oldname and newname
 			 * refer to the same file, so we don't need to check
