@@ -1,16 +1,15 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000-2006
- *	Oracle Corporation.  All rights reserved.
+ * Copyright (c) 2000,2007 Oracle.  All rights reserved.
  *
- * $Id: db_server_proc.c,v 12.14 2006/09/19 13:39:44 mjc Exp $
+ * $Id: db_server_proc.c,v 12.22 2007/06/28 13:59:24 bostic Exp $
  */
 
 #include "db_config.h"
 
 #include "db_int.h"
-#ifndef NO_SYSTEM_INCLUDES
+#ifdef HAVE_SYSTEM_INCLUDE_FILES
 #include <rpc/rpc.h>
 #endif
 #include "db_server.h"
@@ -373,7 +372,8 @@ __env_open_proc(dbenvcl_id, home, flags, mode, replyp)
 	if (__dbsrv_verbose) {
 		dbenv->set_errfile(dbenv, stderr);
 		dbenv->set_errpfx(dbenv, fullhome->home);
-	}
+	} else
+		dbenv->set_errfile(dbenv, NULL);
 
 	/*
 	 * Mask off flags we ignore
@@ -1754,6 +1754,47 @@ err:		FREE_IF_CHANGED(dbp->dbenv, key.data, keydata);
 	return;
 }
 
+/*
+ * PUBLIC: void __db_get_priority_proc __P((u_int, __db_get_priority_reply *));
+ */
+void
+__db_get_priority_proc(dbpcl_id, replyp)
+	u_int dbpcl_id;
+	__db_get_priority_reply *replyp;
+{
+	DB *dbp;
+	ct_entry *dbp_ctp;
+
+	ACTIVATE_CTP(dbp_ctp, dbpcl_id, CT_DB);
+	dbp = (DB *)dbp_ctp->ct_anyp;
+
+	replyp->status =
+	    dbp->get_priority(dbp, (DB_CACHE_PRIORITY *)&replyp->priority);
+}
+
+/*
+ * PUBLIC: void __db_set_priority_proc __P((u_int, u_int32_t,
+ * PUBLIC:      __db_set_priority_reply *));
+ */
+void
+__db_set_priority_proc(dbpcl_id, priority, replyp)
+	u_int dbpcl_id;
+	u_int32_t priority;
+	__db_set_priority_reply *replyp;
+{
+	DB *dbp;
+	ct_entry *dbp_ctp;
+	int ret;
+
+	ACTIVATE_CTP(dbp_ctp, dbpcl_id, CT_DB);
+	dbp = (DB *)dbp_ctp->ct_anyp;
+
+	ret = dbp->set_priority(dbp, (DB_CACHE_PRIORITY)priority);
+
+	replyp->status = ret;
+	return;
+}
+
 /*
  * PUBLIC: void __db_get_re_delim_proc __P((u_int, __db_get_re_delim_reply *));
  */
@@ -2198,12 +2239,12 @@ out:	__os_free(dbenv, jcurs);
 }
 
 /*
- * PUBLIC: void __dbc_c_close_proc __P((u_int, __dbc_c_close_reply *));
+ * PUBLIC: void __dbc_close_proc __P((u_int, __dbc_close_reply *));
  */
 void
-__dbc_c_close_proc(dbccl_id, replyp)
+__dbc_close_proc(dbccl_id, replyp)
 	u_int dbccl_id;
-	__dbc_c_close_reply *replyp;
+	__dbc_close_reply *replyp;
 {
 	ct_entry *dbc_ctp;
 
@@ -2213,14 +2254,14 @@ __dbc_c_close_proc(dbccl_id, replyp)
 }
 
 /*
- * PUBLIC: void __dbc_c_count_proc
- * PUBLIC:     __P((u_int, u_int32_t, __dbc_c_count_reply *));
+ * PUBLIC: void __dbc_count_proc
+ * PUBLIC:     __P((u_int, u_int32_t, __dbc_count_reply *));
  */
 void
-__dbc_c_count_proc(dbccl_id, flags, replyp)
+__dbc_count_proc(dbccl_id, flags, replyp)
 	u_int dbccl_id;
 	u_int32_t flags;
-	__dbc_c_count_reply *replyp;
+	__dbc_count_reply *replyp;
 {
 	DBC *dbc;
 	ct_entry *dbc_ctp;
@@ -2230,7 +2271,7 @@ __dbc_c_count_proc(dbccl_id, flags, replyp)
 	ACTIVATE_CTP(dbc_ctp, dbccl_id, CT_CURSOR);
 	dbc = (DBC *)dbc_ctp->ct_anyp;
 
-	ret = dbc->c_count(dbc, &num, flags);
+	ret = dbc->count(dbc, &num, flags);
 	replyp->status = ret;
 	if (ret == 0)
 		replyp->dupcount = num;
@@ -2238,13 +2279,13 @@ __dbc_c_count_proc(dbccl_id, flags, replyp)
 }
 
 /*
- * PUBLIC: void __dbc_c_del_proc __P((u_int, u_int32_t, __dbc_c_del_reply *));
+ * PUBLIC: void __dbc_del_proc __P((u_int, u_int32_t, __dbc_del_reply *));
  */
 void
-__dbc_c_del_proc(dbccl_id, flags, replyp)
+__dbc_del_proc(dbccl_id, flags, replyp)
 	u_int dbccl_id;
 	u_int32_t flags;
-	__dbc_c_del_reply *replyp;
+	__dbc_del_reply *replyp;
 {
 	DBC *dbc;
 	ct_entry *dbc_ctp;
@@ -2253,20 +2294,20 @@ __dbc_c_del_proc(dbccl_id, flags, replyp)
 	ACTIVATE_CTP(dbc_ctp, dbccl_id, CT_CURSOR);
 	dbc = (DBC *)dbc_ctp->ct_anyp;
 
-	ret = dbc->c_del(dbc, flags);
+	ret = dbc->del(dbc, flags);
 
 	replyp->status = ret;
 	return;
 }
 
 /*
- * PUBLIC: void __dbc_c_dup_proc __P((u_int, u_int32_t, __dbc_c_dup_reply *));
+ * PUBLIC: void __dbc_dup_proc __P((u_int, u_int32_t, __dbc_dup_reply *));
  */
 void
-__dbc_c_dup_proc(dbccl_id, flags, replyp)
+__dbc_dup_proc(dbccl_id, flags, replyp)
 	u_int dbccl_id;
 	u_int32_t flags;
-	__dbc_c_dup_reply *replyp;
+	__dbc_dup_reply *replyp;
 {
 	DBC *dbc, *newdbc;
 	ct_entry *dbc_ctp, *new_ctp;
@@ -2279,7 +2320,7 @@ __dbc_c_dup_proc(dbccl_id, flags, replyp)
 	if (new_ctp == NULL)
 		return;
 
-	if ((ret = dbc->c_dup(dbc, &newdbc, flags)) == 0) {
+	if ((ret = dbc->dup(dbc, &newdbc, flags)) == 0) {
 		new_ctp->ct_dbc = newdbc;
 		new_ctp->ct_type = CT_CURSOR;
 		new_ctp->ct_parent = dbc_ctp->ct_parent;
@@ -2300,13 +2341,13 @@ __dbc_c_dup_proc(dbccl_id, flags, replyp)
 }
 
 /*
- * PUBLIC: void __dbc_c_get_proc __P((u_int, u_int32_t, u_int32_t, u_int32_t,
+ * PUBLIC: void __dbc_get_proc __P((u_int, u_int32_t, u_int32_t, u_int32_t,
  * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, u_int32_t, u_int32_t,
- * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, __dbc_c_get_reply *,
+ * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, __dbc_get_reply *,
  * PUBLIC:     int *));
  */
 void
-__dbc_c_get_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
+__dbc_get_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
     keysize, datadlen, datadoff, dataulen, dataflags, datadata, datasize,
     flags, replyp, freep)
 	u_int dbccl_id;
@@ -2323,7 +2364,7 @@ __dbc_c_get_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
 	void *datadata;
 	u_int32_t datasize;
 	u_int32_t flags;
-	__dbc_c_get_reply *replyp;
+	__dbc_get_reply *replyp;
 	int * freep;
 {
 	DBC *dbc;
@@ -2381,7 +2422,7 @@ __dbc_c_get_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
 		data.flags |= DB_DBT_PARTIAL;
 
 	/* Got all our stuff, now do the get */
-	ret = dbc->c_get(dbc, &key, &data, flags);
+	ret = dbc->get(dbc, &key, &data, flags);
 
 	/*
 	 * Otherwise just status.
@@ -2441,14 +2482,14 @@ err:		FREE_IF_CHANGED(dbenv, key.data, keydata);
 }
 
 /*
- * PUBLIC: void __dbc_c_pget_proc __P((u_int, u_int32_t, u_int32_t, u_int32_t,
+ * PUBLIC: void __dbc_pget_proc __P((u_int, u_int32_t, u_int32_t, u_int32_t,
  * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, u_int32_t, u_int32_t,
  * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, u_int32_t, u_int32_t,
- * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, __dbc_c_pget_reply *,
+ * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, __dbc_pget_reply *,
  * PUBLIC:     int *));
  */
 void
-__dbc_c_pget_proc(dbccl_id, skeydlen, skeydoff, skeyulen, skeyflags,
+__dbc_pget_proc(dbccl_id, skeydlen, skeydoff, skeyulen, skeyflags,
     skeydata, skeysize, pkeydlen, pkeydoff, pkeyulen, pkeyflags, pkeydata,
     pkeysize, datadlen, datadoff, dataulen, dataflags, datadata, datasize,
     flags, replyp, freep)
@@ -2472,7 +2513,7 @@ __dbc_c_pget_proc(dbccl_id, skeydlen, skeydoff, skeyulen, skeyflags,
 	void *datadata;
 	u_int32_t datasize;
 	u_int32_t flags;
-	__dbc_c_pget_reply *replyp;
+	__dbc_pget_reply *replyp;
 	int * freep;
 {
 	DBC *dbc;
@@ -2525,7 +2566,7 @@ __dbc_c_pget_proc(dbccl_id, skeydlen, skeydoff, skeyulen, skeyflags,
 	data.data = datadata;
 
 	/* Got all our stuff, now do the get */
-	ret = dbc->c_pget(dbc, &skey, &pkey, &data, flags);
+	ret = dbc->pget(dbc, &skey, &pkey, &data, flags);
 	/*
 	 * Otherwise just status.
 	 */
@@ -2609,13 +2650,13 @@ err:		FREE_IF_CHANGED(dbenv, skey.data, skeydata);
 }
 
 /*
- * PUBLIC: void __dbc_c_put_proc __P((u_int, u_int32_t, u_int32_t, u_int32_t,
+ * PUBLIC: void __dbc_put_proc __P((u_int, u_int32_t, u_int32_t, u_int32_t,
  * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, u_int32_t, u_int32_t,
- * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, __dbc_c_put_reply *,
+ * PUBLIC:     u_int32_t, void *, u_int32_t, u_int32_t, __dbc_put_reply *,
  * PUBLIC:     int *));
  */
 void
-__dbc_c_put_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
+__dbc_put_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
     keysize, datadlen, datadoff, dataulen, dataflags, datadata, datasize,
     flags, replyp, freep)
 	u_int dbccl_id;
@@ -2632,7 +2673,7 @@ __dbc_c_put_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
 	void *datadata;
 	u_int32_t datasize;
 	u_int32_t flags;
-	__dbc_c_put_reply *replyp;
+	__dbc_put_reply *replyp;
 	int * freep;
 {
 	DB *dbp;
@@ -2669,7 +2710,7 @@ __dbc_c_put_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
 	data.data = datadata;
 
 	/* Got all our stuff, now do the put */
-	ret = dbc->c_put(dbc, &key, &data, flags);
+	ret = dbc->put(dbc, &key, &data, flags);
 
 	*freep = 0;
 	if (ret == 0 && (flags == DB_AFTER || flags == DB_BEFORE) &&
@@ -2686,3 +2727,45 @@ __dbc_c_put_proc(dbccl_id, keydlen, keydoff, keyulen, keyflags, keydata,
 	replyp->status = ret;
 	return;
 }
+
+/*
+ * PUBLIC: void
+ * PUBLIC: __dbc_get_priority_proc __P((u_int, __dbc_get_priority_reply *));
+ */
+void
+__dbc_get_priority_proc(dbccl_id, replyp)
+	u_int dbccl_id;
+	__dbc_get_priority_reply *replyp;
+{
+	DBC *dbc;
+	ct_entry *dbc_ctp;
+
+	ACTIVATE_CTP(dbc_ctp, dbccl_id, CT_CURSOR);
+	dbc = (DBC *)dbc_ctp->ct_anyp;
+
+	replyp->status =
+	    dbc->get_priority(dbc, (DB_CACHE_PRIORITY *)&replyp->priority);
+}
+
+/*
+ * PUBLIC: void __dbc_set_priority_proc __P((u_int, u_int32_t,
+ * PUBLIC:      __dbc_set_priority_reply *));
+ */
+void
+__dbc_set_priority_proc(dbccl_id, priority, replyp)
+	u_int dbccl_id;
+	u_int32_t priority;
+	__dbc_set_priority_reply *replyp;
+{
+	DBC *dbc;
+	ct_entry *dbc_ctp;
+	int ret;
+
+	ACTIVATE_CTP(dbc_ctp, dbccl_id, CT_CURSOR);
+	dbc = (DBC *)dbc_ctp->ct_anyp;
+
+	ret = dbc->set_priority(dbc, (DB_CACHE_PRIORITY)priority);
+
+	replyp->status = ret;
+	return;
+}
