@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 1996-2001
-#	Sleepycat Software.  All rights reserved.
+# Copyright (c) 1996,2007 Oracle.  All rights reserved.
 #
-# $Id: recd003.tcl,v 11.28 2001/08/03 16:39:26 bostic Exp $
+# $Id: recd003.tcl,v 12.5 2007/05/17 15:15:55 bostic Exp $
 #
 # TEST	recd003
 # TEST	Duplicate recovery tests.  For every known duplicate log message,
@@ -34,7 +33,7 @@ proc recd003 { method {select 0} args } {
 	set eflags "-create -txn -home $testdir"
 
 	puts "\tRecd003.a: creating environment"
-	set env_cmd "berkdb env $eflags"
+	set env_cmd "berkdb_env $eflags"
 	set dbenv [eval $env_cmd]
 	error_check_bad dbenv $dbenv NULL
 
